/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(CURSOR_H)
#define CURSOR_H

#include <boost/shared_ptr.hpp>
#include <list>

#include "image.h"

class Cursor
{
public:
  typedef boost::shared_ptr< Cursor > Handle;

  void setCursor(short x, short y, short z, short v);
  void setCursor(short x, short y, short z);

  short inqX() const;
  short inqY() const;
  short inqZ() const;
  short inqV() const;

  /** Create a new cursor
   * @param i the initial image which will follow this cursor
   */
  static Cursor::Handle init(const Image::Handle& i) { 
    return Cursor::Handle( new Cursor(i) ); 
  }

  /** Attach an image to this cursor
   * @param i handle of an image which requires notification of any 
   *          changes to this cursor
   * @see Design Patterns p293
   */
  void attach(const Image::Handle& i);

  /** Detach an image from this cursor
   * @param i handle of the image to be removed from the notification list
   * @see Design Patterns p293
   */
  void detach(const Image::Handle& o);

private:
  Cursor(const Image::Handle& i);

  /* Notify all observers of a change to this cursor
   * @see Design Patterns p293
   */
  void notify();

  short m_x;
  short m_y;
  short m_z;
  short m_v;

  std::list<Image::Handle> m_observers;
};

inline short Cursor::inqX() const { return m_x; }
inline short Cursor::inqY() const { return m_y; }
inline short Cursor::inqZ() const { return m_z; }
inline short Cursor::inqV() const { return m_v; }

#endif
