/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#include "histogram.h"

#include <algorithm>
#include <cmath>

Histogram::Histogram(Volume::Handle v, unsigned int n):
  m_volume(v), m_nBins(n), m_delta(0), m_max(0), m_min(0)
{
  m_bins.reserve(m_nBins);
  m_bins.resize(m_nBins, 0);

  float offset = m_volume->inqMin();
  float range = m_volume->inqMax() - m_volume->inqMin();
  m_delta = range / m_nBins;

  unsigned int nVoxels = m_volume->inqX() * m_volume->inqY() * m_volume->inqZ();

  for(unsigned int voxel = 0; voxel < nVoxels; voxel++)
    {
      unsigned int binNumber = (int)floor((m_volume->value(voxel) - offset) / m_delta);
      if(binNumber < 0) binNumber = 0;
      if(binNumber >= m_nBins) binNumber = m_nBins - 1;
      m_bins[binNumber]++;
    }

  m_min = *std::min_element(m_bins.begin(), m_bins.end());
  m_max = *std::max_element(m_bins.begin(), m_bins.end());
}

Histogram::~Histogram()
{
}

Histogram::Handle Histogram::getHistogram(Volume::Handle v)
{
  return Histogram::Handle(new Histogram(v, 200));
}

float Histogram::inqFrequency(unsigned int n) const { return m_bins[n]; }

unsigned int Histogram::inqBins() const { return m_nBins; }
unsigned int Histogram::inqMin() const { return m_min; }
unsigned int Histogram::inqMax() const { return m_max; }
