/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */



// timeseries.h: interface for the TimeSeries class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(TIMESERIES_H)
#define TIMESERIES_H

#include <avwio/avwio.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>
#include <string>

class TimeSeries
{
public:
  typedef boost::shared_ptr< TimeSeries > Handle;

  virtual float normalized (short nVol) = 0;
  virtual float real       (short nVol) = 0;
  virtual float inqMaxVal  () = 0;
  virtual float inqMinVal  () = 0;
  virtual int   inqVolCount() = 0;

  TimeSeries() {}
  virtual ~TimeSeries() {}
};

template <class VoxelType>
class TimeSeriesStore: public TimeSeries
{
  // friend class TestVolumeStore;

public:
  typedef boost::shared_ptr< TimeSeriesStore<VoxelType> > Handle;

  virtual ~TimeSeriesStore();

	VoxelType& operator()    (short nVol);
  virtual float normalized (short nVol);
  virtual float real       (short nVol);
  virtual int   inqVolCount();
  virtual float inqMaxVal  ();
  virtual float inqMinVal  ();
  
  void setMin(VoxelType min) { m_min = min; }
  void setMax(VoxelType max) { m_max = max; }
  void setMinVal();
  void setMaxVal();
  VoxelType inqMin() const   { return  m_min; }
  VoxelType inqMax() const   { return  m_max; }
  void setVolCount(int n)    { m_count = n; }


  static Handle getTimeSeries(AVW* avw, short x, short y, short z);


private:
  TimeSeriesStore(VoxelType* buffer);
  int m_count;
  VoxelType  m_min, m_max;
  VoxelType  m_minVal, m_maxVal;
  VoxelType* m_buffer;
};

typedef TimeSeriesStore<unsigned char> TimeSeriesB;
typedef TimeSeriesStore<short>         TimeSeriesS;
typedef TimeSeriesStore<int>           TimeSeriesI;
typedef TimeSeriesStore<float>         TimeSeriesF;
typedef TimeSeriesStore<double>        TimeSeriesD;

#include "timeseries.inc"

#endif
