#include "timeseries.h"
#include "avwio/avwio.h"

template <class VoxelType>
TimeSeriesStore<VoxelType>::TimeSeriesStore(VoxelType* buf)
  :  m_buffer(buf)
{
  m_count = 0;
}

template <class VoxelType>
TimeSeriesStore<VoxelType>::~TimeSeriesStore()
{
  delete [] m_buffer;
}

template <class VoxelType>
typename TimeSeriesStore<VoxelType>::Handle TimeSeriesStore<VoxelType>::getTimeSeries(AVW* avw, short x, short y, short z)
{
  short xDim, yDim, zDim, vDim;
  AvwGetDim(avw, &xDim, &yDim, &zDim, &vDim);

  VoxelType* buf = new VoxelType[vDim];

  AvwReadTimeSeries(avw, buf, x, y, z, 1);

  Handle dst(new TimeSeriesStore(buf));

  int min, max;
  AvwGetMinMax(avw, &min, &max);
  dst->setMin(min);
  dst->setMax(max);  
  dst->setVolCount(vDim);
  dst->setMinVal();
  dst->setMaxVal();

  return dst;
}

template <class VoxelType>
void TimeSeriesStore<VoxelType>::setMaxVal()
{
  m_maxVal = *std::max_element(m_buffer,m_buffer + m_count);
}

template <class VoxelType>
void TimeSeriesStore<VoxelType>::setMinVal()
{
  m_minVal = *std::min_element(m_buffer,m_buffer + m_count);
}


