
#include "volume.h"
#include "avwio/avwio.h"
#include <stdio.h>

template <class VoxelType>
VolumeStore<VoxelType>::VolumeStore(short x, short y, short z, VoxelType* buf)
  : m_x(x), m_y(y), m_z(z), m_buffer(buf)
{
}

template <class VoxelType>
VolumeStore<VoxelType>::~VolumeStore()
{
  delete [] m_buffer;
}

template <class VoxelType>
typename VolumeStore<VoxelType>::Handle VolumeStore<VoxelType>::getVolume(AVW* avw, float min, float max, size_t n)
{
  short x, y, z, v;
  AvwGetDim(avw, &x, &y, &z, &v);

  VoxelType* buf = new VoxelType[x * y * z];

  AvwSeekVolume(avw, n);
  AvwReadVolumes(avw, buf, 1);

  Handle dst(new VolumeStore(x, y, z, buf));

  dst->setMin(min);
  dst->setMax(max);

  return dst;
}



template <class VoxelType>
bool VolumeStore<VoxelType>::saveVolume(AVW* avw)
{
  return AvwWriteVolumes(avw, m_buffer, 1);
}

template <class VoxelType>
typename VolumeStore<VoxelType>::Handle VolumeStore<VoxelType>::clone(const Handle& src)
{
  VoxelType* buf = new VoxelType[src->inqX() * src->inqY() * src->inqZ() * sizeof(VoxelType)];
	
  // Copy src's buffer here
  Handle dst(new VolumeStore(src->inqX(), src->inqY(), src->inqZ(), buf));

  return dst;
}

template <class VoxelType>
typename VolumeStore<VoxelType>::Handle VolumeStore<VoxelType>::create(short x, short y, short z, VoxelType* buf)
{
  Handle dst(new VolumeStore(x, y, z, buf));

  return dst;
}

template <class VoxelType>
void VolumeStore<VoxelType>::calculateMinMax()
{
  m_min = *std::min_element(&m_buffer[0], &m_buffer[m_x * m_y * m_z -1]);
  m_max = *std::max_element(&m_buffer[0], &m_buffer[m_x * m_y * m_z -1]);
}

template <class VoxelType>
typename VolumeStore<VoxelType>::Handle VolumeStore<VoxelType>::createBlank(short x,short y,short z)
{
  VoxelType* buf = new VoxelType[x * y * z];


  Handle dst(new VolumeStore(x, y, z, buf));

  unsigned int width  = dst->inqX();
  unsigned int height = dst->inqY();
  unsigned int depth  = dst->inqZ();

  for(unsigned int zl = 0; zl< depth; ++zl)  
    {
      for(unsigned int yl = 0; yl < height; ++yl)
        {            
        for(unsigned int xl = 0; xl < width; ++xl) 
          {
            (*dst)(xl,yl,zl) = 0;
          }
        }
    }

  int min, max;
  min = 0;
  max = 4;
  dst->setMin(min);
  dst->setMax(max);

  return dst;
}



