/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


/****************************************************************************
** $Id: application.h,v 1.30 2003/07/10 14:39:35 jim Exp $
**
** Copyright (C) 2002 University of Oxford.  All rights reserved.
**
*****************************************************************************/

#if !defined (APPLICATION_H)
#define APPLICATION_H

#if defined(WIN32)
#pragma warning(disable:4786)
#endif

#include <qmainwindow.h>
#include "cursor.h"
#include <qlist.h>
#include "imagegroup.h" 
#include "overlaylist.h"

class QLabel;
class QString;
class QStatusBar;
class QScrollView;
class QMultiLineEdit;
class QToolBar;
class QPopupMenu;
class QWorkspace;
class FileError;
class ViewWidget;

class ApplicationWindow: public QMainWindow, 
			 public CursorObserver
{
  Q_OBJECT
public:
  ApplicationWindow();
  virtual ~ApplicationWindow();
  virtual void update(const Cursor::Handle& c);

private slots:

  void load();
  void closeWindow();
  void about();
  void aboutQt();
  void aboutGraphics();
  void fileMenuAboutToShow();  
  void viewMenuAboutToShow();
  void windowsMenuAboutToShow();
  void windowsMenuActivated( int id );
  void displayMessage(const QString &, int);

public slots:
  void saveOverlay();
  void addOverlay();  
  void remOverlay();
  void createMask();
  void addLookUpTable(); 
  void orthoView();
  void lightboxView();
  void movieView();
  void openTimeSeries();  
  void openMultTimeSeries();
  void openHistogram();
 
signals:
  void message(const QString&, int );

private:
  QWorkspace* m_ws;
  QMenuBar* m_mb;
  QPopupMenu* m_fileMenu;
  QPopupMenu* m_windowsMenu;
  QPopupMenu* m_toolsMenu;
  
  int m_fileOpenID,m_fileCloseID,m_fileAddOverlayID,m_fileRemOverlayID,
      m_fileCreateMaskID, m_fileSaveOverlayID;
  int m_imageAddLutID;
  int m_viewOrthoID,m_viewLightID,m_viewMovieID,
      m_viewHistID,m_viewTimeID,m_viewTimeGridID;
  void loadFile(const QString & absFilePath);
  void buildMenus();
  bool checkAbsFilePath(const QString & absFilePath);
  void setupStatusBar();
  void constructToolBar();
  void connectControls();
  bool windowListEmpty();
  void catchFileError(FileError);
  void viewShow(ViewWidget* v);
  OverlayList::Handle activeOverlayList();
  OverlayList::Handle copyActiveOverlayList();

  QString            m_filename;
  QStatusBar        *m_statusBar;

  QPopupMenu        *m_viewMenu;
  QScrollView       *m_sv;

  ImageGroup::Handle m_imageGroup;
  Cursor::Handle     m_cursor;

  bool m_lightBoxEnabled;

};


#endif
