/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(BRICON_H)
#define BRICON_H

#include <memory>
#include <boost/shared_ptr.hpp>

/**
 * @author David Flitney <flitney@fmrib.ox.ac.uk>
 * @date   Thu Jan  2 15:18:22 2003
 * 
 * @brief  Class for managing brightness and contrast changes
 * 
 * A BriCon object tracks modifications to the brightness and contrast settings
 * by updating the range of voxel intensities that are displayable. The adjust
 * method can be called for a given image intensity to find the bricon adjusted
 * intensity.
 */
class BriCon {
public:
  typedef boost::shared_ptr< BriCon > Handle;

  static Handle create(float min, float max) { return Handle(new BriCon(min, max)); }

  float adjust(float v) const;

  void setRange(float min, float max);
  void modifyRange(float deltaBri, float deltaCon);
  void updateRange();
  void reset();
  void setMin(float min);
  void setMax(float max);
  float inqMin() const;
  float inqMax() const;
  Handle clone();

  virtual ~BriCon();

private:
  BriCon(float min, float max);
  BriCon(float origMin, float origMax, float min, float max);

  struct Implementation;  
  const std::auto_ptr<Implementation> m_impl;
  
};

class BriConObserver {
public:
  BriConObserver() {}

  virtual void update(const BriCon *) = 0;

  virtual ~BriConObserver() {}
};

#endif
