/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(BRICONWIDGET_H)
#define BRICONWIDGET_H

#if defined(WIN32)
#pragma warning (disable:4786)
#endif

#include <qwidget.h>
#include <list>

#include "bricon.h"
#include "overlaylist.h"

class QLineEdit;
class QToolButton;
class QSlider;
class BriConWidget;

class BriConWidgetObserver
{
public:

  virtual ~BriConWidgetObserver() {}
  virtual void update(const BriConWidget* w) = 0;

  BriConWidgetObserver() {}
};

class BriConWidget : public QWidget, OverlayListObserver
{
  Q_OBJECT
public:
  BriConWidget(QWidget *parent, OverlayList::Handle list);
  virtual ~BriConWidget();

  virtual void update(const OverlayList* list, OverlayListMsg message);

  void attach(BriConWidgetObserver* o);
  void detach(BriConWidgetObserver* o);
  void notify();

private:
  
  BriCon::Handle m_bricon;
  OverlayList::Handle   m_list;
  bool       m_blockEvents;
  QLineEdit* m_minBox;
  QLineEdit* m_maxBox;
  QSlider*   m_briSlider;
  QSlider*   m_conSlider;
  float      m_originalMin;
  float      m_originalMax;

  std::list<BriConWidgetObserver*> m_observers;
  
private slots:

  void reset();
  void minChanged(const QString& valueText);
  void maxChanged(const QString& valueText);
  void briSliderChanged(int value);
  void conSliderChanged(int value);
  void sliderReleased();
};

#endif
