/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include <algorithm>

#include "cursor.h"

Cursor::Cursor(CursorObserver *o): m_x(0), m_y(0), m_z(0), m_v(0)
{
  attach(o);
}

Cursor::Cursor(short x, short y, short z, short v):m_x(x), m_y(y),m_z(z),m_v(v)
{

}

void Cursor::setCursor(short x, short y, short z)
{
  m_x = x;
  m_y = y;
  m_z = z;

  notify();
}

void Cursor::setCursorRepaint(short x, short y, short z)
{
  m_x = x;
  m_y = y;
  m_z = z;

  m_repaint = true;
  notify();
  m_repaint = false;
}

void Cursor::repaint()
{
  m_repaint = true;
  notify();
  m_repaint = false;
}

Cursor::Handle Cursor::clone()
{
  return Handle(new Cursor(m_x, m_y, m_z, m_v));
}

void Cursor::setCursor(short x, short y, short z, short v)
{
  m_v = v;
  setCursor(x, y, z);
}

void Cursor::attach(CursorObserver *o)
{
  m_observers.push_back(o);
}

void Cursor::detach(CursorObserver *o)
{
  m_observers.remove(o);
}

struct Update
{
  Update(Cursor::Handle c): m_cursor(c) {}

  void operator()(CursorObserver *v)
  {
    v->update(m_cursor);
  }

  const Cursor::Handle m_cursor;
};

void Cursor::notify() const
{
  std::for_each(m_observers.begin(), m_observers.end(), 
                Update(countedThis()));
}


