/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include <qlineedit.h>
#include <qspinbox.h>
#include <qtoolbutton.h>
#include <qpixmap.h>
#include <qvalidator.h>
#include <qtooltip.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <algorithm>
#include <stdio.h>

#include "undo.xpm"
#include "tracker.h"
#include "drawwidget.h"

const char * sizeBoxText = "Pen size.<br><hr>Changes the dimensions of the masking pen.";
const char *  valBoxText = "Pen value<br><hr>Controls the value that is written when drawing a mask.";
const char *  undoButtonText = "Undo button<br><hr>Reverts mask back to state before last edit.";
DrawWidget::DrawWidget(QWidget* w) :   QWidget(w),m_size(1),m_value(1),m_undoPressed(false)
{

  m_valBox = new QSpinBox(this);
  m_valBox->setValidator(new QIntValidator(this));
  m_valBox->setValue(1);
  m_valBox->setRange(0,255);
  
  connect(m_valBox,SIGNAL(valueChanged(int)),
          this,SLOT(valBoxChanged(int)));
  
  m_sizeBox = new QSpinBox(this);
  m_sizeBox->setValidator(new QIntValidator(this));
  m_sizeBox->setValue(1);
  m_sizeBox->setRange(1,20);

  connect(m_sizeBox,SIGNAL(valueChanged(int)),
          this,SLOT(sizeBoxChanged(int)));
 
  m_undoButton = new QToolButton(this);
  m_undoButton->setPixmap(QPixmap(undo));
  m_undoButton->setMaximumSize(QPixmap(undo).size());
  
  connect(m_undoButton, SIGNAL(pressed()),this, SLOT(undoButtonPressed())); 

  QToolTip::add(m_valBox,     tr(valBoxText));
  QToolTip::add(m_sizeBox,    tr(sizeBoxText));
  QToolTip::add(m_undoButton, tr(undoButtonText));
 
  QHBoxLayout* hlayout = new QHBoxLayout(this);

  hlayout->addWidget(m_valBox);
  hlayout->addWidget(m_sizeBox);
  hlayout->addWidget(m_undoButton);

  show();
}

DrawWidget::~DrawWidget()
{

}


void DrawWidget::attach(DrawWidgetObserver* o)
{
  m_observers.remove(o);
  m_observers.push_back(o);
}

void DrawWidget::detach(DrawWidgetObserver* o)
{
  m_observers.remove(o);
}

struct Update
{
  Update(DrawWidget* l): m_ol(l) {}

  void operator()(DrawWidgetObserver* v)
  {
    v->update(m_ol);
  }

  DrawWidget* m_ol;
};

void DrawWidget::notify()
{
    std::for_each(m_observers.begin(), m_observers.end(), Update(this));
}

void DrawWidget::sizeBoxChanged(int value)
{
  m_size = m_sizeBox->value();
}

void DrawWidget::valBoxChanged(int value)
{
  m_value = m_valBox->value();
}

void DrawWidget::undoButtonPressed()
{
  m_undoPressed = true;
  notify();
  m_undoPressed = false;
}
