/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "graphplot.h"
#include "graphwidget.h"
#include <algorithm>
#include <math.h>

TimeSeriesPlot::TimeSeriesPlot(TimeSeries::Handle t):m_timeSeries(t),m_active(false)
{
  m_volCount = m_timeSeries->inqVolCount(); 
  setMinMaxVals();
}
TimeSeriesPlot::~TimeSeriesPlot(){}

TimeSeriesPlot::Handle TimeSeriesPlot::getTimeSeriesPlot(TimeSeries::Handle t)
{
  Handle dst(new TimeSeriesPlot(t));
  
  return dst;
}

void TimeSeriesPlot::drawPlot(GraphWidget* g)
{
  if(m_active){g->setPenColor( QWidget::darkRed );}
  else        {g->setPenColor( QWidget::darkGreen );}

  for(int n = 1; n < m_volCount; ++n)
      g->drawLine(n-1,m_timeSeries->real(n-1),n,m_timeSeries->real(n));
}

float TimeSeriesPlot::inqMinYVal(){return m_minYVal;}
float TimeSeriesPlot::inqMaxYVal(){return m_maxYVal;}
float TimeSeriesPlot::inqMinXVal(){return m_minXVal;}
float TimeSeriesPlot::inqMaxXVal(){return m_maxXVal;}

void TimeSeriesPlot::setMinMaxVals()
{
  m_minYVal = m_timeSeries->inqMinVal();
  m_maxYVal = m_timeSeries->inqMaxVal();
  m_minXVal = 0;
  m_maxXVal = m_volCount;
}

HistogramPlot::HistogramPlot(Histogram::Handle h): m_histogram(h)
{
  setMinMaxVals();
}

HistogramPlot::~HistogramPlot() {}

HistogramPlot::Handle HistogramPlot::getHistogramPlot(Histogram::Handle h)
{
  Handle dst(new HistogramPlot(h));

  return dst;
}

void HistogramPlot::drawPlot(GraphWidget* g)
{
  g->setPenColor( QWidget::darkGreen );

  for(unsigned int n = 1; n < m_histogram->inqBins(); ++n)
    g->drawLine(n-1,log10(std::max(1.0f, m_histogram->inqFrequency(n-1))),
		n,  log10(std::max(1.0f, m_histogram->inqFrequency(n)  )));      
}

float HistogramPlot::inqMinYVal(){return m_minYVal;}
float HistogramPlot::inqMaxYVal(){return m_maxYVal;}
float HistogramPlot::inqMinXVal(){return m_minXVal;}
float HistogramPlot::inqMaxXVal(){return m_maxXVal;}

void HistogramPlot::setMinMaxVals()
{
  m_minYVal = log10(std::max(1u, m_histogram->inqMin()));
  m_maxYVal = log10(std::max(1u, m_histogram->inqMax()));
  m_minXVal = 0;
  m_maxXVal = m_histogram->inqBins();
}

