/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(GRAPHPLOT_H)
#define GRAPHPLOT_H

#include <boost/shared_ptr.hpp>
#include "storage/timeseries.h"
#include "storage/histogram.h"

class GraphWidget;

class GraphPlot 
{
 public:
  typedef boost::shared_ptr< GraphPlot > Handle;
  GraphPlot(){}
  virtual ~GraphPlot(){}
  virtual void drawPlot(GraphWidget* g) = 0;
  virtual float inqMinXVal() = 0;
  virtual float inqMaxXVal() = 0;
  virtual float inqMinYVal() = 0;
  virtual float inqMaxYVal() = 0; 
  virtual void  setActive (bool state) = 0;
};

class TimeSeriesPlot : public GraphPlot
{
 public:
  typedef boost::shared_ptr< TimeSeriesPlot> Handle;
  static Handle getTimeSeriesPlot(TimeSeries::Handle t);
  virtual  ~TimeSeriesPlot();
  virtual void drawPlot(GraphWidget* g);  
  virtual void setActive(bool state){m_active = state;}
  virtual float inqMinXVal();
  virtual float inqMaxXVal();
  virtual float inqMinYVal();
  virtual float inqMaxYVal();


 private:
  TimeSeriesPlot(TimeSeries::Handle t);
  TimeSeries::Handle m_timeSeries;
  void setMinMaxVals();
  bool m_active;

  float m_minXVal, m_maxXVal, m_minYVal, m_maxYVal, m_volCount;


};

class HistogramPlot : public GraphPlot
{
public:
  typedef boost::shared_ptr< HistogramPlot> Handle;

  static Handle getHistogramPlot(Histogram::Handle h);

  virtual  ~HistogramPlot();
  virtual void drawPlot(GraphWidget* g);  
  virtual void setActive(bool state){m_active = state;}
  virtual float inqMinXVal();
  virtual float inqMaxXVal();
  virtual float inqMinYVal();
  virtual float inqMaxYVal();

 private:
  HistogramPlot(Histogram::Handle h);
  Histogram::Handle m_histogram;
  void setMinMaxVals();
  bool m_active;

  float m_minXVal, m_maxXVal, m_minYVal, m_maxYVal;
};
  
#endif
