/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(GRAPHWIDGET_H)
#define GRAPHWIDGET_H


#include "graphplot.h"
#include <qwidget.h>
#include <qpainter.h>
#include <qfont.h>
#include <vector>

class Rect;
class GraphWindow;
class QPainter;
class QString;
class ViewWidget;
class GraphDim;

class GraphWidget : public QWidget
{
  Q_OBJECT 

public:
	GraphWidget(QWidget* parent, QString xAxis, QString yAxis);
	~GraphWidget();    	
	void addPlot(GraphPlot::Handle plot);
  void remPlot(int n);
  void remAllPlots();
  GraphPlot::Handle getPlot(int n);
  int  inqPlotCount();
  void drawCircle(float x,float y,int radius);
	void drawLine(float xA, float yA, float xB, float yB);
	void setAxisMaxMin(float xMin, float xMax, float yMin, float yMax);
	void setFontSizes(int fontSizeLabels, int fontSizeNums);
  void setPenColor(QColor color){m_penColor = color;}
  GraphDim* m_gd;
protected:
	virtual void paintEvent( QPaintEvent* );

private slots:

private:	
  QString m_xAxisLabel, m_yAxisLabel;	
  QColor m_penColor; 
	float m_xAxisMax, m_xAxisMin, m_yAxisMax, m_yAxisMin,m_xStep,m_yStep; 
 	int		m_fontSizeLabels,	m_fontSizeNums;//Size of font for labels and numbers.
  QFont m_labelFont,m_numFont;

	QPainter 				m_paint;                              //Shared painting object.
  std::vector<GraphPlot::Handle> m_plots;                      
  typedef std::vector<GraphPlot::Handle>::iterator PlotsIt;
 void findDataMaxMin();
	void drawAxis();
	void drawChecks();
  void drawLabels();
  int convX(float x);
  int convY(float y);
  float findStepSize(float range);
  void autoRange(float min, float max,float &newMin, float &newMax, float &step);
  int findDp(float step);
  void findFontSizes(int &,int &,int &,int &);
};



#endif
