/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "gridserieswidget.h"
#include "graphwidget.h"
#include "storage/timeseries.h"
#include <qlayout.h>

class DrawGraph {
public:
  DrawGraph(ImageGroup::Handle i,  QGridLayout* g,bool a,short x,short y, short z) : 
            m_ig(i), m_grid(g),m_active(a), m_x(x), m_y(y), m_z(z), m_row(0), m_col(0){}
  void operator()(GraphWidget* g);

private:
  ImageGroup::Handle  m_ig;
  QGridLayout*        m_grid;
  bool                m_active;
  short m_x, m_y, m_z;
  int   m_row, m_col;
  
};

void DrawGraph::operator()(GraphWidget* g)
{
  TimeSeries::Handle ts =  m_ig->getMainImage()->getTimeSeries
    (m_x + m_col - 1, m_y + m_row - 1,m_z);

  TimeSeriesPlot::Handle plot = TimeSeriesPlot::getTimeSeriesPlot(ts);
  
  if(m_col == 1 && m_row == 1 && m_active)
    plot->setActive(true);

  g->remAllPlots();
 
  g->addPlot(plot);    

  m_grid->addWidget(g,m_col,m_row);  

  ++m_col;
  if(m_col > 2){m_col = 0;++m_row;}
}

GridSeriesWidget::GridSeriesWidget(QWidget *parent, ImageGroup::Handle i,bool a):
  QWidget(parent),m_imageGroup(i),m_active(a),m_x(0), m_y(0), m_z(0)
{
  setMinimumSize(300,300);
  m_grid = new QGridLayout(this,3,3);
  createGraphs();
}

GridSeriesWidget::~GridSeriesWidget(){};

void GridSeriesWidget::setTimeSeries(short x, short y, short z)
{
  m_x = x;  m_y = y;  m_z = z;  
  drawGraphs(); 
}



void GridSeriesWidget::createGraphs()
{

  for(int r = 0;r <3;++r)
  {
    for(int s = 0;s < 3;++s)
    {  
        GraphWidget* g = new GraphWidget(this,"","");  
        m_grid->addWidget(g,r,s);
        m_graphList.push_back(g);
    }
  }

}



void GridSeriesWidget::drawGraphs()
{
  std::for_each(m_graphList.begin(),m_graphList.end(),
                DrawGraph(m_imageGroup,m_grid,m_active,m_x,m_y,m_z));
}

 

