/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(GROUP_H)
#define GROUP_H

#include <boost/shared_ptr.hpp>
#include <list>

// This class implements the Observer pattern (Design Patterns pp293-303
template <class Groupable>
class Group {
public:
  typedef boost::shared_ptr<Group> Handle;

  ~Group();

  Group::Handle createGroup();

  void addChild(const Groupable::Handle&);
  void removeChild(const Groupable::Handle&);

  void notify();
  void update(Groupable::Handle&) = 0;

private:
  Group();

  std::list<Groupable::Handle> m_children;
};

class CursorGroup: public Group<Cursor>
{
  void setCursor
};

class ImageGroup: public Group<Image>
{
};

#endif
