/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "histogramwidget.h"
#include "qwt_plot.h"
#include "storage/histogram.h"
#include <math.h>

class Plot: public QwtPlot
{
public:
  Plot(QWidget *parent, const Volume::Handle &volume): QwtPlot(parent)
  {
    setTitle("Image Histogram");
    
    setAxisTitle(xBottom, "Intensity");
    setAxisTitle(yLeft,   "log #voxels");

    long curve = insertCurve("Intensity histogram");
    setCanvasBackground(QColor(white));
    setCurvePen(curve, QPen(blue));

    setAxisOptions(QwtPlot::yLeft, QwtAutoScale::Logarithmic);
    setAxisMargins(QwtPlot::yLeft, 0, 0);

    const unsigned int nBins = 200;

    double x[nBins];
    double y[nBins];

    volume->calculateMinMax();

    float offset = volume->inqMin();
    float range = volume->inqMax() - volume->inqMin();
    float delta = range / nBins;

    unsigned int nVoxels = volume->inqX() * volume->inqY() * volume->inqZ();

    for(unsigned int n = 0; n < nBins; n++)
      {
	x[n] = offset + (n * delta);
	y[n] = 0.1;
      }

    for(unsigned int voxel = 0; voxel < nVoxels; voxel++)
      {
	unsigned int binNumber = (int)floor((volume->value(voxel) - offset) / delta);
	if(binNumber < 0) binNumber = 0;
	if(binNumber >= nBins) binNumber = nBins - 1;
	y[binNumber]++;
      }

    setCurveData(curve, x, y, nBins);
    replot();
  }
};

/** 
 * Constructor
 * 
 * @param parent Parent widget
 * @param i The applications image group.
 */
HistogramWidget::HistogramWidget(QWidget *parent, ImageGroup::Handle i): 
  ViewWidget(parent), m_imageGroup(i)
{
  m_graphWidget = new Plot(this, m_imageGroup->getMainImage()->getVolume(0));
  setMinimumSize(400,400);
  m_graphWidget->show();
  setCentralWidget(m_graphWidget);
}

HistogramWidget::~HistogramWidget() { } 

