/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if defined(WIN32)
#pragma warning(disable:4786)
#endif

#include "lookuptable.h"
#include "imagedisplaysetting.h"

struct ImageDisplaySetting::Implementation
{
  Implementation(LookUpTable::Handle lut,float trans, bool visible, Image::Handle image): 
    m_transparency(trans), 
    m_isVisible(visible), 
    m_lookUpTable(lut) 
  {
    Volume::Handle v = image->getVolume(0);

    float min = image->getInfo()->inqMin();
    float max = image->getInfo()->inqMax();

    if(min == max) {v->calculateMinMax();}

    m_bricon = BriCon::create(v->inqMin(), v->inqMax());
  }

  Implementation(LookUpTable::Handle lut,
                 float trans, 
                 bool visible, 
                 BriCon::Handle bricon): 
    m_transparency(trans), 
    m_isVisible(visible), 
    m_lookUpTable(lut),
    m_bricon(bricon){}

  float               m_transparency;
  bool                m_isVisible;
  LookUpTable::Handle m_lookUpTable;
  BriCon::Handle      m_bricon;
};

ImageDisplaySetting::Handle ImageDisplaySetting::create(Image::Handle image,
                                                        LookUpTable::Handle lut,
                                                        float trans, 
                                                        bool visible)
{
  return Handle(new ImageDisplaySetting(image, lut, trans, visible));
}

ImageDisplaySetting::ImageDisplaySetting(Image::Handle image, 
                                         LookUpTable::Handle lut, 
                                         float trans, 
                                         bool visible) :
  m_impl(new Implementation(lut, trans, visible, image))
{
}

ImageDisplaySetting::ImageDisplaySetting(BriCon::Handle bricon, 
                                         LookUpTable::Handle lut, 
                                         float trans, 
                                         bool visible) :
  m_impl(new Implementation(lut, trans, visible, bricon))
{
}

ImageDisplaySetting::~ImageDisplaySetting()
{
}

void ImageDisplaySetting::setTransparency(float trans)
{
  m_impl->m_transparency = trans;
}

void ImageDisplaySetting::setVisibility(bool visible)
{
  m_impl->m_isVisible = visible;
}

void ImageDisplaySetting::setLookUpTable(LookUpTable::Handle lut)
{
  m_impl->m_lookUpTable = lut;
}


float ImageDisplaySetting::inqTransparency()
{
  return m_impl->m_transparency;
}

bool ImageDisplaySetting::inqVisibility()
{
  return m_impl->m_isVisible;
}

LookUpTable::Handle  ImageDisplaySetting::inqLookUpTable()
{
  return  m_impl->m_lookUpTable;
}

BriCon::Handle  ImageDisplaySetting::inqBriCon()
{
  return  m_impl->m_bricon;
}

ImageDisplaySetting::Handle  ImageDisplaySetting::clone()
{
  return
    Handle(new ImageDisplaySetting(m_impl->m_bricon->clone(),
                                   m_impl->m_lookUpTable,
                                   m_impl->m_transparency,
                                   m_impl->m_isVisible));

}
