/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(IMAGEWIDGET_H)
#define IMAGEWIDGET_H

#include "drawwidget.h"
#include "viewwidget.h"
#include "overlaylist.h"
#include "shape.h"

class QToolBar;
class QToolButton;
class QSpinBox;
class QCheckBox;
class OverlayWidget;
class BriConWidget;
class CursorWidget;
class DrawWidget;
class ImageGroup;
class OverlayDialog;

typedef std::list<MetaImage> MetaImageList;
typedef MetaImageList::iterator MetaImageListIt;

class ImageWidget : public ViewWidget, public DrawWidgetObserver,
                    public OverlayListObserver
{
  Q_OBJECT

public:
  ImageWidget(QWidget* parent,ImageGroup::Handle i,
              OverlayList::Handle);
  ImageWidget(QWidget* parent,ImageGroup::Handle i,
              OverlayList::Handle,Cursor::Handle c);
  ~ImageWidget();  
  OverlayList::Handle getOverlayList();  
  void update(const DrawWidget* w);
  void update(const OverlayList* i, OverlayListMsg msg);
  inline int inqX();
  inline int inqY();
  inline int inqZ();

public slots:
  void setPanMode();
  void setZoomMode(); 
  void setCursorMode();
  void setMaskMode();
  void setCrossHairMode();
  void setZoomCombo(int);  
  void setVolSpinBox(int);
  void openOverlayDialog();
  void volChanged(int n);

private slots: 
  void maskButtonToggled(bool);

signals:
  virtual void modeChanged(int n) = 0;
  virtual void zoomChanged(int n) = 0;
  virtual void resetZoom() = 0;
  virtual void crossHairModeChanged(bool);

private:
  void connectControls();
  void constructToolBar(); 
  void loadOverlaysList();
  void constructor();
  void undoGraphics();
  void clearUndoList();

  QSpinBox            *m_zoomSpinBox;
  QSpinBox            *m_volSpinBox;

  ImageGroup::Handle   m_imageGroup; 

  int m_xDim;
  int m_yDim; 
  int m_zDim;

protected:
  QToolButton         *m_noneModeButton;
  Cursor::Handle       m_cursor;
  QToolButton         *m_zoomModeButton;
  QToolButton         *m_cursorModeButton;
  QToolButton         *m_panModeButton;
  QToolButton         *m_maskModeButton;
  QToolButton         *m_resetButton;
  QCheckBox           *m_crossHairCheckBox;
  OverlayList::Handle  m_overlayList;  
  OverlayWidget       *m_overlayWidget;
  BriConWidget        *m_briconWidget;
  CursorWidget        *m_cursorWidget;
  DrawWidget          *m_drawWidget;
  OverlayDialog       *m_overlayDialog;
  QToolBar            *m_toolbar; 
  std::list<Shape::Handle>        m_undoList;
};

int ImageWidget::inqX(){return m_xDim;}
int ImageWidget::inqY(){return m_yDim;}
int ImageWidget::inqZ(){return m_zDim;}
#endif
