/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(LOOKUPTABLE_H)
#define LOOKUPTABLE_H

#include <boost/shared_ptr.hpp>
#include <string>
//#include <GL/gl.h>
#include <qgl.h>
#include <vector>
#include "stdio.h"

typedef GLubyte ColorTriplet[3];
typedef GLubyte ColorRGBA[4];

/**
 * @author James Saunders <jim@fmrib.ox.ac.uk>
 * @date   Mon Dec 23 17:59:36 2002
 * 
 * @brief  Implementation of a color look up table (lut).
 * 
 * Provides a color lut suitable for GL rendering along with methods
 * to create standard luts and read custom ones from file.
 */
class LookUpTable
{
public:
  typedef boost::shared_ptr< LookUpTable > Handle;

  static LookUpTable::Handle load(const std::string& filename); 

  static LookUpTable::Handle red();
  static LookUpTable::Handle blue();
  static LookUpTable::Handle green();
  static LookUpTable::Handle yellow();
  static LookUpTable::Handle redYellow();
  static LookUpTable::Handle blueLightblue();
  static LookUpTable::Handle render1();
  static LookUpTable::Handle render1t();
  static LookUpTable::Handle render2();
  static LookUpTable::Handle render2t();
  static LookUpTable::Handle render3();

  void pushValue(GLubyte red, GLubyte green, GLubyte blue, int index);
  const ColorTriplet& inqValue(float f);  
  const ColorTriplet& inqValueIndex(float f);

  void allocateMemory(int size);  
  std::string inqLutName() const;
  bool isIndexLut();
  virtual ~LookUpTable();

private:   
  LookUpTable(const std::string& filename);
  LookUpTable();
  LookUpTable::Handle LoadStdLut(const char ** lutData,
                                 LookUpTable::Handle lut,
                                 int elementCount,
                                 const std::string  name);
  std::string extractName(std::string filename);

  ColorTriplet*  m_lookUpData;
  ColorTriplet   m_blackTriplet;
  int            m_lookUpDataSize;
  std::string    m_fileName;
  std::string    m_lutName;
  bool           m_isIndexLut;
};


inline std::string LookUpTable::inqLutName() const { return m_lutName; }

inline bool isValidLut(const LookUpTable::Handle & lut)
{
  if(!lut.get()){return false;}
  else          {return true;}
}

inline bool LookUpTable::isIndexLut()
{
  return m_isIndexLut;
}
  
#endif
