/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(MDIWINDOW_H)
#define MDIWINDOW_H

#include <qmainwindow.h>

#include "imagegroup.h"
#include "cursor.h"

class ViewWidget;
class QStatusBar;
class QScrollView;
class QSpinBox;
class QLabel;
class QSplitter;
class QWorkspace;

class MDIWindow: public QMainWindow, public CursorObserver
{
  Q_OBJECT
public:
  MDIWindow( QWidget* parent, const char* name, int wflags );
  ~MDIWindow();

  void load( const QString& fn );
  void save();
  void saveAs();

  virtual void update(const Cursor* c);

signals:
  void message(const QString&, int );
  void volumeChanged(int n);
  void zoomChanged(int n);
  void modeChanged(int m);

public slots:
  void setVolume(int);
  void setZoom(int);
  void setCursorMode();
  void setBriconMode();
  void setZoomMode();
  void addOverlay();
  void orthoView();
  void openTimeSeries();
  void lightboxView();
  void detailDialog();
  void showAnalysisArea();
private:
  void buildMenus();
  void setupStatusBar();
  void constructToolBar();
  void connectControls();
  void setView(ViewWidget* newView);
  void openAnalysisArea();
  void closeAnalysisArea();
  QLabel            *m_cursorLabel;
  QString            m_filename;
  QStatusBar        *m_statusBar;
  QSpinBox          *m_volSpinBox;
  QSpinBox          *m_zoomSpinBox;
  ViewWidget        *m_view;
  ViewWidget        *m_graphView;
  QPopupMenu        *m_viewMenu;
  QScrollView       *m_sv;
  QScrollView       *m_svB;
  QWorkspace        *m_ws;
  QSplitter         *m_splitter;
  ImageGroup::Handle m_imageGroup;
  Cursor            *m_cursor;
  int                m_menuIDAnalysis;
};

#endif
