/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#if defined(WIN32)
#pragma warning(disable:4786)
#endif

#include "moviewidget.h"

#include <qtimer.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qlayout.h>
#include "overlaywidget.h"
#include "tracker.h"

#include "movie.xpm"

static const char * movieText = "Movie mode.<br><hr>"
"Click to start/stop the movie";

MovieWidget::MovieWidget(QWidget *parent, ImageGroup::Handle i,OverlayList::Handle ol) :  
  ImageWidget(parent,i,ol), m_image(i)
{
  TRACKER("MovieWidget::MovieWidget");

  m_slice = SliceWidget::Handle(new AxialWidget(this, "axial", m_cursor, 
                                                m_overlayList, m_briconWidget, 
                                                m_drawWidget, m_undoList));
  setCentralWidget(m_slice.get());

  connect(this, SIGNAL(modeChanged(int)), m_slice.get(), SLOT(setMode(int)));
  connect(this, SIGNAL(zoomChanged(int)), m_slice.get(), SLOT(setZoom(int)));
  connect(this, SIGNAL(resetZoom()),      m_slice.get(), SLOT(resetZoom()));
 
  ImageInfo::Handle info(m_image->getMainImage()->getInfo());
  if(info->inqNumVolumes() > 1)
    {
      m_timer = new QTimer(this);
    }

  m_timer = new QTimer( this );
  QToolBar *movieBar = new QToolBar(this, "");
  QToolButton *movieButton = new QToolButton(QPixmap(movie_xpm), tr(movieText),
					     QString::null, this, SLOT(toggleMovie()),
					     movieBar, "movie on/off");
  movieButton->setToggleButton(true);
  addToolBar(movieBar, Top, FALSE); 

  connect( m_timer, SIGNAL(timeout()), this, SLOT(nextFrame()) );
}

MovieWidget::~MovieWidget()
{
  TRACKER("MovieWidget::~MovieWidget");
  m_timer->stop();
}

void MovieWidget::toggleMovie()
{
  if(!m_timer->isActive())
    m_timer->start(50, false);
  else
    m_timer->stop();
}

void MovieWidget::update(const Cursor::Handle& c)
{
  TRACKER("MovieWidget::update(Cursor)");
  

  m_slice->setImageCursor(c->inqX(), c->inqY(), c->inqZ(), c->inqV());
}

void MovieWidget::nextFrame()
{
  int n = (m_cursor->inqZ() + 1) % m_image->inqZ(); 
  m_cursor->setCursorRepaint(m_cursor->inqX(),m_cursor->inqY(),n);
}
