/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(OVERLAYLIST_H)
#define OVERLAYLIST_H

#include "imagegroup.h"
#include <boost/shared_ptr.hpp>
typedef enum {Select, Visibility, Transparency, Order,  
              Add,    Rem ,       LookUpTable,  Security} OverlayListMsg;

class OverlayListObserver;



class OverlayList :  public ImageGroupObserver
{
public:
  typedef boost::shared_ptr< OverlayList > Handle;
  typedef boost::weak_ptr< OverlayList > WeakHandle;
  
  Handle clone();
  static Handle create(ImageGroup::Handle i);

  ~OverlayList();
  const MetaImage&    getMainMetaImage() const;
  const Image::Handle getMainImage() const; 
  const MetaImage     getActiveMetaImage() const;
  void setActiveMetaImage(const MetaImage& mi);
  void setTransparency(float trans);
  void setLookUpTable(LookUpTable::Handle lut);
  void setVisibility(bool state);
  void setReadOnly(bool state);
  void setOverlays(std::list<MetaImage>);
  void setMainMetaImage(MetaImage &);
  void moveOverlayUp();
  void moveOverlayDown();
  const MetaImageListIt begin();
  const MetaImageListIt end(); 

  void update(const ImageGroup* i, ImageGroupMsg message);
  
  void attach(OverlayListObserver* o);
  void detach(OverlayListObserver* o);
  void notify(OverlayListMsg message);
  
  Image::Handle inqActiveImage();
  inline int inqX();
  inline int inqY();
  inline int inqZ();

private:  
  
  OverlayList(ImageGroup::Handle i);
  void loadOverlaysList();
  void assignLatestLUT();
  
  std::list<MetaImage> m_overlays;
  MetaImage            m_mainMetaImage;
  ImageGroup::Handle   m_imageGroup;    
  int m_xDim;
  int m_yDim; 
  int m_zDim;
  int m_currentLut;
  MetaImage            m_activeMetaImage;
  std::list<OverlayListObserver*> m_observers;

};

int OverlayList::inqX(){return m_xDim;}
int OverlayList::inqY(){return m_yDim;}
int OverlayList::inqZ(){return m_zDim;}

class OverlayListObserver
{
 
public:

  virtual ~OverlayListObserver() {}
  virtual void update(const OverlayList* list, OverlayListMsg message) = 0;

  OverlayListObserver() {}
};

inline bool isValidOverlayList(const OverlayList::Handle ol)
{
  if(!ol.get()){return false;}
  else         {return true;}
}

#endif
