/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined (OVERLAYWIDGET_H)
#define OVERLAYWIDGET_H

#include <algorithm>
#include <qwidget.h>
#include <qlistview.h>
#include "overlaylist.h"


class QCheckBox;
class QSlider;
class QPushButton;
class LayerListItem;
class ImageWidget;


class OverlayWidget : public QWidget, OverlayListObserver
{

 Q_OBJECT

 public:
  OverlayWidget(QWidget* w,OverlayList::Handle l);
  virtual ~OverlayWidget();
  void update(const OverlayList* l, OverlayListMsg msg);
 private:  
  void updateListView();
  void updateListItem();
  void setAllInActive(); 
  void updateUpDownButtons();
  void setVisibility(bool state);
  void updateControls();
  QListViewItem* getLayerItem(const MetaImage &mi)const;
  

  OverlayList::Handle   m_overlayList; 
  QCheckBox*     m_visibleButton;
  QCheckBox*     m_unlockButton;
  QListView*     m_layersListView;
  QSlider*       m_transSlider;
  QPushButton*   m_upButton;
  QPushButton*   m_downButton;

  bool           m_blockEvents;
  bool           m_blockSliderUpdate;
  bool           m_sliderDisabled;

 signals:
   virtual void infoButtonAction();

 private slots:

     void listSelectChanged();
     void visibleButtonChanged(bool state);
     void unlockButtonChanged(bool state);
 
     void listDoubleClicked(QListViewItem* item);

     void transSliderChanged(int value);
     void upButtonPressed();
     void downButtonPressed();
     void detailsButtonPressed();
};


#endif
