/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  CCOPYRIGHT */

#include "timeserieswidget.h"
#include "graphwidget.h"
#include "cursor.h"
#include "storage/timeseries.h"
#include "graphplot.h"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qtoolbar.h>
#include <qspinbox.h>
#include <qtoolbutton.h>
#include <qtooltip.h>

#include "add.xpm"
#include "rem.xpm"


const char * addSeriesText = "Add series.<br><hr>Press to add the timeseries of the current cursor position.";

const char * remSeriesText = "Remove series.<br><hr>Press to remove the currently selected time series.";

TimeSeriesWidget::TimeSeriesWidget(QWidget *parent, ImageGroup::Handle& i, Cursor::Handle& c):
  ViewWidget(parent), m_imageGroup(i),m_cursor(c)
{
  setMinimumSize(200,200);
  m_graphWidget = new GraphWidget(this,"Volumes","Value");

  QPixmap addIcon = QPixmap( add );
  QPixmap remIcon = QPixmap( rem );

  QToolBar *tb1 = new QToolBar(this, "");  

  new QToolButton( addIcon, tr( addSeriesText ), QString::null,this, SLOT(addTimeSeries()), tb1, "Add time series" );
  new QToolButton( remIcon, tr( remSeriesText ), QString::null,this, SLOT(remTimeSeries()), tb1, "Remove time series" );
 
  m_seriesSpinBox = new QSpinBox(0,0,1,tb1);
  m_seriesSpinBox->setValue(1);
  m_seriesSpinBox->setButtonSymbols(QSpinBox::PlusMinus);
  m_seriesSpinBox->setSpecialValueText(" ");
  QToolTip::add( m_seriesSpinBox, tr("Time series selection"));
  
  setCentralWidget(m_graphWidget);
  addToolBar(tb1,Top,FALSE);
 
  connect( m_seriesSpinBox, SIGNAL(valueChanged(int)), this, SLOT(setActivePlot(int)) );
}

TimeSeriesWidget::~TimeSeriesWidget(){};

void TimeSeriesWidget::setImageGroup(const ImageGroup::Handle& i)
{
  m_imageGroup = i;
}

void TimeSeriesWidget::addTimeSeries()
{
   TimeSeries::Handle timeSeries =  m_imageGroup->getMainImage()->getTimeSeries(m_cursor->inqX(),m_cursor->inqY(),m_cursor->inqZ());
  TimeSeriesPlot::Handle plot = TimeSeriesPlot::getTimeSeriesPlot(timeSeries);
 

  m_graphWidget->addPlot(plot);
  updateSeriesSpinBox();

}
void TimeSeriesWidget::remTimeSeries()
{
  int n = m_seriesSpinBox->value();
  if(n)
    {
      m_graphWidget->remPlot(n - 1);
      updateSeriesSpinBox();
    }
}

void TimeSeriesWidget::updateSeriesSpinBox()
{
  m_seriesSpinBox->setMaxValue(m_graphWidget->inqPlotCount());
  m_seriesSpinBox->setValue(m_graphWidget->inqPlotCount());
}

void TimeSeriesWidget::setActivePlot(int n)
{
  int count = m_graphWidget->inqPlotCount();
  GraphPlot::Handle g;

  for(int i = 0;i<count; ++i)
    {
      g = m_graphWidget->getPlot(i);
      if(i==(n-1)){g->setActive(true);}
      else        {g->setActive(false);}
    }
  m_graphWidget->update();

}
