/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#if !defined(__AVWIO_H)
#define __AVWIO_H

#include <stdio.h>
#include "dbh.h"

#define AVW_READ  0
#define AVW_WRITE 1

#ifdef __cplusplus
extern "C" {
#endif

typedef struct 
{
  FILE *imgfp;
  FILE *hdrfp;

  int mode, sameorder;

  struct dsr header;
} AVW;

  AVW *AvwOpen(const char *filename, const char *opts);

void AvwInitHeader(AVW *avw, short t, 
		   size_t x, size_t y, size_t z, size_t v,
		   float vx, float vy, float vz, float tr,
		   size_t dim,
		   const char* units);
void AvwCloneHeader(AVW *dest, const AVW *src);

size_t AvwReadVolumes(AVW *avw, void *buffer, size_t nvols);
size_t AvwWriteVolumes(AVW *avw, const void *buffer, size_t nvols);
size_t AvwReadSliceSeries(AVW *avw, void *buffer,short slice, size_t nvols);
size_t AvwReadRowSeries(AVW *avw, void *buffer, short row, short slice, size_t nvols);
size_t AvwReadTimeSeries(AVW * avw, void *buffer, short xVox, short yVox, short zVox, size_t nvols);
size_t AvwWriteSliceSeries(AVW *avw,const  void *buffer,short slice, size_t nvols);
size_t AvwWriteRowSeries(AVW *avw,const  void *buffer, short row, short slice, size_t nvols);
size_t AvwReadCplxVolumes(AVW *avw, void *buffer, size_t nvols, char mode);
size_t AvwWriteplxVolumes(AVW *avw, void *buffer, size_t nvols, char mode);  
int AvwSeekVolume(AVW *avw, size_t vols);

size_t AvwGetVolSize(AVW *avw);

void AvwSetDim(AVW *avw, short x, short y, short z, short v);
void AvwGetDim(AVW *avw, short *x, short *y, short *z, short *v);
void AvwSetDimensionality(AVW *avw, size_t dim);
void AvwGetDimensionality(AVW *avw, size_t *dim);
void AvwSetVoxDim(AVW *avw, float x, float y, float z, float tr);
void AvwGetVoxDim(AVW *avw, float *x, float *y, float *z, float *tr);
void AvwSetMinMax(AVW *avw, int min, int max);
void AvwGetMinMax(AVW *avw, int *min, int *max);
void AvwGetCalMinMax(AVW *avw, float *min, float *max);
void AvwSetCalMinMax(AVW *avw, float  min, float  max);
void AvwGetAuxFile(AVW *avw,char *aux_file);
void AvwSetAuxFile(AVW *avw,const char *aux_file);
void AvwSetVoxUnits(AVW *avw, const char *units);
void AvwGetVoxUnits(AVW *avw, char *units);
void AvwSetDataType(AVW *avw, short t);
size_t AvwGetDataType(AVW *avw, short *t);
void AvwSetOrientation(AVW *avw, const char orient);
void AvwGetOrientation(AVW *avw, char *orient);
void AvwSetOriginator(AVW *avw, const short orig[5]);
void AvwGetOriginator(AVW *avw, short orig[5]);

int IsBackwardOrder(void);
void Reverse2(void *ptr, size_t n);
void Reverse4(void *ptr, size_t n);
void AvwReverseDsr(AVW *avw);
void AvwReverseBuffer(AVW *avw, void *buffer, size_t n);


  int AvwClose(AVW *avw);

#ifdef __cplusplus
}
#endif

#endif
