/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#include <stdlib.h>
#include <string.h>
#include "avwio.h"

void AvwInitHeader(AVW *avw, short t, 
		   size_t x, size_t y, size_t z, size_t v,
		   float vx, float vy, float vz, float tr,
		   size_t dim,
		   const char* units)
{
  short *sbuf;

  avw->sameorder = 1;

  avw->header.hk.sizeof_hdr=348; /* number of bytes */
  strcpy(avw->header.hk.data_type,"");		
  avw->header.hk.extents=16384;
  avw->header.hk.session_error=0;
  avw->header.hk.regular='r';
  avw->header.hk.hkey_un0=' ';
  strcpy(avw->header.dime.cal_units,"");
  avw->header.dime.unused1=0;
  avw->header.dime.dim_un0=0;
  avw->header.dime.vox_offset=0.00;
  avw->header.dime.funused1=1.00;
  avw->header.dime.funused2=0.00;
  avw->header.dime.funused3=0.00;
  avw->header.dime.cal_max=0.00;
  avw->header.dime.cal_min=0.00;
  avw->header.dime.compressed=0;
  avw->header.dime.verified=0;
  strcpy(avw->header.hist.descrip,"Externally created avw->header");
  strcpy(avw->header.hist.aux_file,"");
  AvwSetOrientation(avw, 0);

  sbuf = (short *) calloc(5, sizeof(short));
  AvwSetOriginator(avw, sbuf);
  free(sbuf);

  strcpy(avw->header.hist.generated,"");
  strcpy(avw->header.hist.scannum,"");
  strcpy(avw->header.hist.patient_id,"");
  strcpy(avw->header.hist.exp_date,"");
  strcpy(avw->header.hist.exp_time,"");
  strcpy(avw->header.hist.hist_un0,"");
  avw->header.hist.views=0;
  avw->header.hist.vols_added=0;
  avw->header.hist.start_field=0;
  avw->header.hist.field_skip=0;
  avw->header.hist.omax=0;
  avw->header.hist.omin=0;
  avw->header.hist.smax=0;
  avw->header.hist.smin=0;  

  switch(t)
    {
    case DT_UNSIGNED_CHAR:
      AvwSetMinMax(avw, 0, 255);
      break;
    case DT_SIGNED_SHORT:
      AvwSetMinMax(avw, -32767, 32767);
      break;
    case DT_FLOAT:
      AvwSetMinMax(avw, -10, 10);
      break;
    case DT_COMPLEX:
      AvwSetMinMax(avw, -32767, 32767);
      break;
    }

  AvwSetDim(avw, x, y, z, v);
  AvwSetDimensionality(avw, dim);
  AvwSetVoxDim(avw, vx, vy, vz, tr);
  AvwSetVoxUnits(avw, units);
  AvwSetDataType(avw, t);
}

