/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "avwio.h"
/**
   Function for opening AVW files
   @author Dave Flitney
   @param filename The root name for the AVW file
   @param opts "w", "r" or "r+" opens for writing, reading or overwrite
   @return The AVW structure
*/
AVW *AvwOpen(const char *filename, const char *opts)
{
  AVW *avw;
  char imgfname[1024], hdrfname[1024], *bopts=calloc(1024,1);
  int i;
  
  if((avw = (AVW *) calloc(1, sizeof(AVW))) == NULL)
    {
      perror("AvwOpen: Error while reserving memory for AVW!");
      return NULL;
    }

  sprintf(imgfname, "%s.img", filename);
  sprintf(hdrfname, "%s.hdr", filename);

  /* add in 'b' for windows compatibility */
  for(i=0;i<strlen(opts);i++)
    if(opts[i]!='b' && opts[i]!='t')
      strncat(bopts,opts+i,1);
  strcat(bopts,"b");

  if((avw->hdrfp = fopen(hdrfname, bopts)) == NULL)
    {
      perror("AvwOpen: Error while opening header!");
      free(avw);
      return NULL;
    }
  
  if((avw->imgfp = fopen(imgfname, bopts)) == NULL)
    {
      perror("AvwOpen: Error while opening image!");
      free(avw);
      return NULL;
    }

  if(opts[0] == 'r')
    {
      if(!fread(&avw->header, sizeof(struct dsr), 1, avw->hdrfp)) 
	{
	  perror("AvwOpen: Error while reading header!");
	  return NULL;
	}

      (opts[1] == '+') ? (avw->mode = AVW_WRITE) : (avw->mode = AVW_READ);
    }
  else
    {
      avw->mode = AVW_WRITE;
      AvwInitHeader(avw, DT_SIGNED_SHORT, 
		    1, 1, 1, 0,
		    0.0, 0.0, 0.0, 0.0,
		    4,
		    "mm");
    }

  if( !(avw->sameorder = (avw->header.hk.sizeof_hdr == 348)) ) 
    {
      AvwReverseDsr(avw);
      assert(avw->header.hk.sizeof_hdr == 348);
    }

  return avw;     
}
