/*  FSLView - 2D/3D Interactive Image Viewer

    James Saunders, David Flitney and Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2002-2003 University of Oxford  */

/*  Part of FSLView 1.0
    http://www.fmrib.ox.ac.uk/fsl
    fsl@fmrib.ox.ac.uk
    
    Developed at FMRIB (Oxford Centre for Functional Magnetic Resonance
    Imaging of the Brain), Department of Clinical Neurology, Oxford
    University, Oxford, UK
    
    LICENCE

FSLView v1.0 sources are released under the terms of the GPL
(http://www.gnu.org/copyleft/gpl.html). FSLView binaries use Qt
(http://www.trolltech.com) which is released under GPL for X11 and
under the "Qt non-commercial licence" for Windows.  The FSLView
binaries are released under the GPL, with the added exception that we
also give permission to link this program with the Qt non-commercial
edition, and distribute the resulting executable, without including
the source code for the Qt non-commercial edition in the source
distribution.

FSLView is distributed "AS IS" under this Licence in the hope that it
will be useful, but in order that the University as a charitable
foundation protects its assets for the benefit of its educational and
research purposes, the University makes clear that no condition is
made or to be implied, nor is any warranty given or to be implied, as
to the accuracy of the Software, or that it will be suitable for any
particular purpose or for use under any specific conditions.
Furthermore, the University disclaims all responsibility for the use
which is made of the Software. It further disclaims any liability for
the outcomes arising from using the Software.
    
 */


#include <stdlib.h>
#include <string.h>
#include "avwio.h"

size_t AvwGetVolSize(AVW *avw)
{
  short x, y, z, v;

  AvwGetDim(avw, &x, &y, &z, &v);
  return x * y * z;
}

void AvwSetDim(AVW *avw, short x, short y, short z, short v)
{
  avw->header.dime.dim[1] = x;
  avw->header.dime.dim[2] = y;
  avw->header.dime.dim[3] = z;
  avw->header.dime.dim[4] = v;
  avw->header.dime.dim[5] = 0;
  avw->header.dime.dim[6] = 0;
  avw->header.dime.dim[7] = 0;
}

void AvwGetDim(AVW *avw, short *x, short *y, short *z, short *v)
{
  *x = avw->header.dime.dim[1];
  *y = avw->header.dime.dim[2];
  *z = avw->header.dime.dim[3];
  *v = avw->header.dime.dim[4];
}

void AvwGetAuxFile(AVW *avw,char *aux_file)
{
  strncpy(aux_file,avw->header.hist.aux_file, 24);
  aux_file[23] = '\0';
}

void AvwSetAuxFile(AVW *avw,const char *aux_file)
{
  strncpy(avw->header.hist.aux_file, aux_file, 24);
}

void AvwSetDimensionality(AVW *avw, size_t dim)
{
    avw->header.dime.dim[0] = dim;
}

void AvwGetDimensionality(AVW *avw, size_t *dim)
{
  *dim = avw->header.dime.dim[0];
}

void AvwSetVoxDim(AVW *avw, float x, float y, float z, float tr)
{
  avw->header.dime.pixdim[0]=0.00;
  avw->header.dime.pixdim[1]=x;
  avw->header.dime.pixdim[2]=y;
  avw->header.dime.pixdim[3]=z;
  avw->header.dime.pixdim[4]=tr;
  avw->header.dime.pixdim[5]=0.00;
  avw->header.dime.pixdim[6]=0.00;
  avw->header.dime.pixdim[7]=0.00;
}

void AvwGetVoxDim(AVW *avw, float *x, float *y, float *z, float *tr)
{
  *x = avw->header.dime.pixdim[1];
  *y = avw->header.dime.pixdim[2];
  *z = avw->header.dime.pixdim[3];
  *tr = avw->header.dime.pixdim[4];
}

void AvwSetVoxUnits(AVW *avw, const char *units)
{
  strcpy(avw->header.dime.vox_units, units);
}

void AvwGetVoxUnits(AVW *avw, char *units)
{
  strcpy(units, avw->header.dime.vox_units);
}

void AvwSetDataType(AVW *avw, short t)
{
  avw->header.dime.datatype=t;
  switch(t)
    {
    case DT_UNSIGNED_CHAR:
      avw->header.dime.bitpix=8;
      break;
    case DT_SIGNED_SHORT:
      avw->header.dime.bitpix=16;
      break;
    case DT_SIGNED_INT:
      avw->header.dime.bitpix=32;
      break;
    case DT_FLOAT:
      avw->header.dime.bitpix=32;
      break;
    case DT_DOUBLE:
      avw->header.dime.bitpix=64;
      break;
    case DT_COMPLEX:
      avw->header.dime.bitpix=64;
      break;
    }      
}

size_t AvwGetDataType(AVW *avw, short *t)
{
  short datatype;
  size_t bitpix=0;

  *t = avw->header.dime.datatype;

  /* start MJ hack */
  datatype = avw->header.dime.datatype;
  switch(datatype)
    {
    case DT_UNSIGNED_CHAR:
      bitpix = 8;
      break;
    case DT_SIGNED_SHORT:
      bitpix = 16;
      break;
    case DT_SIGNED_INT:
      bitpix = 32;
      break;
    case DT_FLOAT:
      bitpix = 32;
      break;
    case DT_DOUBLE:
      bitpix = 64;
      break;
    case DT_COMPLEX:
      bitpix = 64;
      break;
    default:
      fprintf(stderr, "Unknown data type of %d\n",datatype);
      perror("AvwGetDataType: unkown datatype in header file!");
    }      

   return bitpix;

  /* old version - pre MJ hack */
  /* return avw->header.dime.bitpix; */
}

void AvwSetOrientation(AVW *avw, const char orient)
{
  avw->header.hist.orient = orient;
}

void AvwGetOrientation(AVW *avw, char *orient)
{
  *orient = avw->header.hist.orient;
}

void AvwSetOriginator(AVW *avw, const short orig[5])
{
  memcpy((void *) avw->header.hist.originator, (void *) orig, 5*sizeof(short));
}

void AvwGetOriginator(AVW *avw, short orig[5])
{
  memcpy((void *) orig, (void *) avw->header.hist.originator, 5*sizeof(short));
}

void AvwSetMinMax(AVW *avw, int min, int max)
{
  avw->header.dime.glmin = min;
  avw->header.dime.glmax = max;
}

void AvwGetMinMax(AVW *avw, int *min, int *max)
{
  *min = avw->header.dime.glmin;
  *max = avw->header.dime.glmax;
}

void AvwSetCalMinMax(AVW *avw, float min, float max)
{
  avw->header.dime.cal_min = min;
  avw->header.dime.cal_max = max;
}

void AvwGetCalMinMax(AVW *avw, float *min, float *max)
{
  *min = avw->header.dime.cal_min;
  *max = avw->header.dime.cal_max;
}

int IsBackwardOrder(void)
{
  int val=1;
  char tmp, *ptr = (char *) &val;
  tmp = *ptr;
  if (tmp==1)  return 1;
  tmp = *(ptr + 3);
  if (tmp==1)  return 0;
  fprintf(stderr, "Cannot determine byte ordering on this machine");
  return 0;
}

void Reverse2(void *ptr, size_t n)
{
  char tmp, *cptr = (char *) ptr;
  size_t i;

  for (i = 0; i < n; i++) {
    tmp = *cptr;
    *cptr = *(cptr + 1);
    *(cptr + 1) = tmp;
    cptr += 2;
  }
}

void Reverse4(void *ptr, size_t n)
{
  char tmp1, tmp2, *cptr = (char *) ptr;
  size_t i;

  for (i = 0; i < n; i++) {
    tmp1 = *cptr;
    tmp2 = *(cptr + 1);
    *cptr = *(cptr + 3);
    *(cptr + 1)= *(cptr + 2);
    *(cptr + 2)= tmp2;
    *(cptr + 3)= tmp1;
    cptr += 4;
  }
}


void Reverse8(void *ptr, size_t n)
{
  char tmp1, tmp2, tmp3, tmp4, *cptr = (char *) ptr;
  size_t i;

  for (i = 0; i < n; i++) {
    tmp1 = *cptr;
    tmp2 = *(cptr + 1);
    tmp3 = *(cptr + 2);
    tmp4 = *(cptr + 3);
    *cptr = *(cptr + 7);
    *(cptr + 1)= *(cptr + 6);
    *(cptr + 2)= *(cptr + 5);
    *(cptr + 3)= *(cptr + 4);
    *(cptr + 4)= tmp4;
    *(cptr + 5)= tmp3;
    *(cptr + 6)= tmp2;
    *(cptr + 7)= tmp1;
    cptr += 8;
  }
}


void AvwReverseDsr(AVW *avw)
{
  char *ptr;

  ptr = (char *) &(avw->header.hk);
  Reverse4(ptr, 1);		/* sizeof_hdr */
  ptr += 32;
  Reverse4(ptr, 1);		/* extents */
  ptr += 4;
  Reverse2(ptr, 1);		/* session_error */
  
  ptr = (char *) &(avw->header.dime);
  Reverse2(ptr, 8);		/* dims */
  ptr += 28;
  Reverse2(ptr, 4);		/* unused1, datatype, bitpix, dim_un0 */
  ptr += 8;
  Reverse4(ptr, 18);		/* pixdim, vox_offset, ... */
				/* cal_min, compressed, ... glmin */

  ptr = (char *) &(avw->header.hist);
  ptr += 105;
  Reverse2(ptr, 5);		/* originator (used to store origin) */
  ptr += 63;
  Reverse4(ptr, 8);		/* views, ... smin */
}

void AvwReverseBuffer(AVW *avw, void *buffer, size_t n)
{
  short datatype;
  size_t voxelbytes;
  
  voxelbytes = AvwGetDataType(avw, &datatype) / (8 * sizeof(char));

  switch(datatype)
    {
    case DT_UNSIGNED_CHAR:
      break;			/* nothing to do */
    case DT_SIGNED_SHORT:
      Reverse2(buffer, n);
      break;
    case DT_SIGNED_INT:
      Reverse4(buffer, n);
      break;
    case DT_FLOAT:
      Reverse4(buffer, n);
      break;
    case DT_DOUBLE:
      Reverse8(buffer, n);
      break;
    case DT_COMPLEX:
      Reverse4(buffer, n * 2);
      break;
    default:
      fprintf(stderr, "Unknown data type!\n");
      fprintf(stderr, "AvwReverseBuffer: cannot reverse buffer");
    }      
}
