
/*  libpic - collection of image display and rendering routines

    Stephen Smith & Christian Beckmann, FMRIB Image Analysis Group

    Copyright (C) 1999-2002 University of Oxford  */

/*  CCOPYRIGHT */


#ifndef __MISCPIC_h
#define __MISCPIC_h

#include "newimage/newimageall.h"
#include "miscmaths/miscmaths.h"
#include <stdarg.h>
#include "gd.h"
#include <vector.h>

#ifndef FALSE
#define FALSE false
#endif
#ifndef TRUE
#define TRUE true
#endif


using namespace NEWIMAGE;

namespace MISCPIC{
  
  //template <class T>
  class miscpic
    {
    public:

      //constructor
      miscpic(){
	nlut = 0;      
	compare= 0;
	trans= -10;
	if(getenv("FSLDIR")!=0){
	  lutbase = string(getenv("FSLDIR")) + "/etc/luts/";
	}
	else{
	  lutbase = string("/");
	}
	title = string("");
	cbartype = string("");
	cbarptr = NULL;
	outim = NULL;
	picr = NULL;
	picg = NULL;
	picb = NULL;
      };

      ~miscpic(){
	if(picr!=NULL) free(picr);
	if(picg!=NULL) free(picg);
	if(picb!=NULL) free(picb);
	if(cbarptr) gdImageDestroy(cbarptr);
	if(outim) gdImageDestroy(outim);
      }

      //  int slicer(volume<T> vol1, volume<T> vol2, char *opts, 
      //		 volumeinfo *volinf, bool debug = FALSE);
      int slicer(volume<float> vol1, volume<float> vol2, char *opts, 
      		 volumeinfo *volinf, bool debug = FALSE);

      //inline int slicer(volume<T> vol1, char *opts, volumeinfo *volinf, 
      //		bool debug = FALSE)

      inline int slicer(volume<float> vol1, char *opts, volumeinfo *volinf, 
			bool debug = FALSE)
	{volume<float> tmp(1,1,1); //volume<float> tmp(1,1,1);
	return this->slicer(vol1, tmp, opts, volinf, debug);}
 

      //inline int slicer(volume<T> vol1, char *opts, bool debug = FALSE)
      inline int slicer(volume<float> vol1, char *opts, bool debug = FALSE)
	{volume<float> tmp(1,1,1);//volume<float> tmp(1,1,1);
	return this->slicer(vol1, tmp, opts, NULL, debug);}
   
      //inline int slicer(volume<T> vol1, volume<T> vol2,char *opts, 
      inline int slicer(volume<float> vol1, volume<float> vol2,char *opts, 
			bool debug = FALSE)
	{return this->slicer(vol1, vol2, opts, NULL, debug);}

      
      int write_png ( char *filename, int x_size, int y_size,	 
 		      unsigned char *r, unsigned char *g, unsigned char *b); 
      int write_pgm ( char *filename, int x_size, int y_size,	
		      unsigned char *i );
      int write_ppm ( char *filename, int x_size, int y_size,	
		      unsigned char *r, unsigned char *g, unsigned char *b );
      void write_pic(char *fname, int width, int height);
      
      inline void set_title(string what) { title = what;}
      inline void set_cbar(string what) { cbartype = what;}

      inline void read_lut(char *fname){
	lut = string(fname); this->read_lut();
      }

      void read_lut();
      
      inline void set_minmax(float bgmin, float bgmax, float s1min,
		  float s1max){
	this->set_minmax(bgmin, bgmax, s1min, s1max, (float)0.0, float(0.0));
      };

      void set_minmax(float bgmin, float bgmax, float s1min,
		  float s1max, float s2min, float s2max);
      
      //inline int overlay(volume<T>& newvol,
      inline int overlay(volume<float>& newvol, volume<float>& bg, volume<float>& s1,
			 volume<float>& s2, float bgmin, float bgmax, float s1min,
			 float s1max, float s2min, float s2max, int colour_type,
			 int checker, volumeinfo *volinf, 
			 bool out_int = false, bool debug = false){
	return this->overlay(newvol, bg, s1, s2, bgmin, bgmax, s1min, 
			     s1max, s2min, s2max,
			     colour_type, checker, volinf, 
			     string(""), string(""), out_int, debug);
      }

      //inline int overlay(volume<T>& newvol,
      int overlay(volume<float>& newvol, volume<float>& bg, volume<float>& s1,
		  volume<float>& s2, float bgmin, float bgmax, float s1min,
		  float s1max, float s2min, float s2max, int colour_type,
		  int checker, volumeinfo *volinf, 
		  string cbarfname, string cbartype, bool out_int = false, 
		  bool debug = false);

    private:

      int x_size, y_size, z_size, size, x_size_pic, y_size_pic, 
	  z_size_pic, nlut, compare, trans;

      bool debug;

      string lut, lutbase, title, cbartype;

      gdImagePtr cbarptr, outim;

      vector<int> rlut, glut, blut; //stores the lut;
      unsigned char *picr, *picg, *picb;

      //volume<T>  inp1, inp2, imr, img, imb;	  
      volume<float>  inp1, inp2, imr, img, imb;
      vector<float> minmax; //will store min and max values for bg and stats images
      //needed for colorbar

      void sag(float xx, int p, int width);
      void cor(float yy, int p, int width);
      void axi(float zz, int p, int width);
            
      int create_cbar(string cbartype);
      int write_cbar(string fname, string cbartype);
      int add_cbar(string cbartype);

      int add_title(int width);
    };
}

#endif
