/*  costfns.h

    Mark Jenkinson, FMRIB Image Analysis Group

    Copyright (C) 2001 University of Oxford  */

/*  CCOPYRIGHT  */


#if !defined(__costfns_h)
#define __costfns_h

#include "newimageall.h"

#ifndef NO_NAMESPACE
 using namespace NEWIMAGE;

 namespace NEWIMAGE {
#endif

  enum costfns { Woods, CorrRatio, MutualInfo, NormCorr, NormMI, LeastSq, 
		 NormCorrSinc };

  class Costfn {
  public:
    const volume<float> &refvol;
    const volume<float> &testvol;
    const volume<float> &rweight;
    const volume<float> &tweight;
  private:
    int *bindex;
    int no_bins;
    ColumnVector plnp;
    int *jointhist;
    int *marghist1;
    int *marghist2;
    float *fjointhist;
    float *fmarghist1;
    float *fmarghist2;
    int p_count;
    costfns p_costtype;
    bool validweights;
  public: 
    float smoothsize;
    float fuzzyfrac;

  public:
    // Publicly available calls
    Costfn(const volume<float>& refv, const volume<float>& inputv);
    Costfn(const volume<float>& refv, const volume<float>& inputv,
	   const volume<float>& refweight, const volume<float>& inweight);
    ~Costfn();

    void set_costfn(const costfns& costtype) { p_costtype = costtype; }
    void set_no_bins(int n_bins);
    int count() { return p_count; }

    // General cost function call
    float cost(const Matrix& affmat);    // affmat is voxel to voxel

  private:
    // Prevent default behaviours
    Costfn();
    Costfn operator=(const Costfn&);
    Costfn(const Costfn&);

    // Internal functions available
    float normcorr(const Matrix& aff); 
    float normcorr_smoothed(const Matrix& aff);
    float normcorr_smoothed_sinc(const Matrix& aff);
    float normcorr_fully_weighted(const Matrix& aff,
				  const volume<float>& refweight, 
				  const volume<float>& testweight);
    
    float leastsquares(const Matrix& aff);
    float leastsquares_smoothed(const Matrix& aff);
    float leastsquares_fully_weighted(const Matrix& aff, 
				      const volume<float>& refweight, 
				      const volume<float>& testweight);
    
    float woods_fn(const Matrix& aff); 
    float woods_fn_smoothed(const Matrix& aff); 
    
    float corr_ratio(const Matrix& aff); 
    float corr_ratio_smoothed(const Matrix& aff); 
    float corr_ratio_fully_weighted(const Matrix& aff,
				    const volume<float>& refweight, 
				    const volume<float>& testweight);
    
    float mutual_info(const Matrix& aff);
    float mutual_info_smoothed(const Matrix& aff);
    float mutual_info_fully_weighted(const Matrix& aff,
				     const volume<float>& refweight, 
				     const volume<float>& testweight);
    
    float normalised_mutual_info(const Matrix& aff);
    float normalised_mutual_info_smoothed(const Matrix& aff);
    float normalised_mutual_info_fully_weighted(const Matrix& aff,
						const volume<float>& refweight, 
						const volume<float>& testweight);
    
    float cost(const Matrix& affmat,
	       const volume<float>& refweight, 
	       const volume<float>& testweight);
    
  };


   //////////////////////////////////////////////////////////////////////////


#ifndef NO_NAMESPACE
 }
#endif

#endif







