/* {{{ Copyright etc. */

/*  avwcc.c - cross-correlate two time series

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2001-2002 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwcc <first_input> <second_input> [cc_thresh]\n\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc, argv)
  int   argc;
  char  *argv [];
{
  /* {{{ vars */

image_struct im1, im2;
int x, size, t1, t2;
double score, *ss1, *ss2, thresh=0.1;

/* }}} */

  /* {{{ process args and input images */

if (argc<3)
     usage();

avw_read(argv[1],&im1);
avw_read(argv[2],&im2);

size=im1.x*im1.y*im1.z;

ss1=malloc(sizeof(double)*im1.t);
ss2=malloc(sizeof(double)*im2.t);

if (argc==4)
     thresh=atof(argv[3]);

/* }}} */
  /* {{{ sum-of-squares LUTs */
 
  for(t1=0;t1<im1.t;t1++)
    {
      score=0;
      for(x=0;x<size;x++)
	score+=im1.i[t1*size+x]*im1.i[t1*size+x];
      ss1[t1]=sqrt(score);
    }

  for(t2=0;t2<im2.t;t2++)
    {
      score=0;
      for(x=0;x<size;x++)
	score+=im2.i[t2*size+x]*im2.i[t2*size+x];
      ss2[t2]=sqrt(score);
    }

/* }}} */

  for(t1=0;t1<im1.t;t1++)
    {
      for(t2=0;t2<im2.t;t2++)
	{
	  score=0;
	  for(x=0;x<size;x++)
	    score+=im1.i[t1*size+x]*im2.i[t2*size+x];
	  score=fabs(score)/(ss1[t1]*ss2[t2]);
	  if (score>thresh)
	    printf("%3d %3d %.2f\n",t1+1,t2+1,score);
	}
    }

  return(0);
}

/* }}} */
