/* {{{ Copyright etc. */

/*  avwcreatehd.c - Create AVW header

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2001 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwcreatehd <xsize> <ysize> <zsize> <tsize> <xvoxsize> <yvoxsize> <zvoxsize> <tr> <xorigin> <yorigin> <zorigin> <datatype> <headername> [-r]\n\n");
  printf("[-r] : reverse byte order (to match byte order in data)\n\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc,argv) 
     int argc;
     char **argv;
{
  char filename[10000], filename2[10000];
  struct dsr hdr;
  FILE *fd;
  static int DataTypeSizes[32] = {0,1,8,0,16,0,0,0,32,0,0,0,0,0,0,0,32};

  if (argc<2)
    usage();

  /* {{{ set uninteresting defaults */

hdr.hk.sizeof_hdr=348;
hdr.hk.data_type[0]=0;
hdr.hk.db_name[0]=0;
hdr.hk.extents=0;
hdr.hk.session_error=0;
hdr.hk.regular='r';
hdr.hk.hkey_un0=0;

sprintf(hdr.dime.vox_units,"mm");
hdr.dime.cal_units[0]=0;
hdr.dime.unused1=0;
hdr.dime.dim_un0=0;
hdr.dime.vox_offset=0;
hdr.dime.funused1=1;
hdr.dime.funused2=0;
hdr.dime.funused3=0;
hdr.dime.compressed=0;
hdr.dime.verified=0;

hdr.hist.descrip[0]=0;
hdr.hist.generated[0]=0;
hdr.hist.scannum[0]=0;
hdr.hist.patient_id[0]=0;
hdr.hist.exp_date[0]=0;
hdr.hist.exp_time[0]=0;
hdr.hist.hist_un0[0]=0;
hdr.hist.views=0;
hdr.hist.vols_added=0;
hdr.hist.start_field=0;
hdr.hist.field_skip=0;
hdr.hist.omax=0;
hdr.hist.omin=0;
hdr.hist.smax=0;
hdr.hist.smin=0;

/* }}} */
  /* {{{ set rest of parameters */

  hdr.dime.dim[0]=4;
  hdr.dime.dim[1]=atoi(argv[1]);
  hdr.dime.dim[2]=atoi(argv[2]);
  hdr.dime.dim[3]=atoi(argv[3]);
  hdr.dime.dim[4]=atoi(argv[4]);
  hdr.dime.dim[5]=0;
  hdr.dime.dim[6]=0;
  hdr.dime.dim[7]=0;

  hdr.dime.pixdim[0]=0;
  hdr.dime.pixdim[1]=atof(argv[5]);
  hdr.dime.pixdim[2]=atof(argv[6]);
  hdr.dime.pixdim[3]=atof(argv[7]);
  hdr.dime.pixdim[4]=atof(argv[8]);
  hdr.dime.pixdim[5]=0;
  hdr.dime.pixdim[6]=0;
  hdr.dime.pixdim[7]=0;

  {
    short short_array[100];
    short_array[0]=atoi(argv[9]);
    short_array[1]=atoi(argv[10]);
    short_array[2]=atoi(argv[11]);
    memcpy(hdr.hist.originator,short_array,5*sizeof(short));
  }

  hdr.hist.orient=0;
  hdr.hist.aux_file[0]=0;

  hdr.dime.glmin=0;
  hdr.dime.glmax=0;
  hdr.dime.cal_min=0;
  hdr.dime.cal_max=0;

  hdr.dime.datatype=atoi(argv[12]);
  hdr.dime.bitpix=DataTypeSizes[hdr.dime.datatype];

/* }}} */
  /* {{{ swap bytes */

if(argc>14)
     swap_hdr(&hdr);

/* }}} */
  /* {{{ write header */

strcpy(filename,argv[13]);

if ( (strcmp(".hdr",filename+strlen(filename)-4)==0) ||
     (strcmp(".img",filename+strlen(filename)-4)==0) )
     filename[strlen(filename)-4]=0;

sprintf(filename2,"%s.hdr",filename);

if((fd=fopen(filename2,"wb"))==NULL)
{
  fprintf(stderr,"Can't open %s\n",filename2);
  exit(1);
}

if ( fwrite(&hdr,1,sizeof(struct dsr),fd) < 1 )
{
  fprintf(stderr,"Couldn't write %s\n",filename2);
  exit(1);
}

fclose(fd);

/* }}} */

  exit(0);
}

/* }}} */
