/* {{{ Copyright etc. */

/*  avwinterleave.c - combine two interleaved frames

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2001 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwinterleave <in1> <in2> <out> [-i]\n\n");
  printf("-i : reverse slice order\n\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc, argv)
  int   argc;
  char  *argv [];
{
  /* {{{ vars */

image_struct im, im2;
FDT  *out;
int  i, z, xysize, size, invert=0;

/* }}} */

  /* {{{ args */

  if (argc<4)
    usage();

  if (argc==5)
    invert=1;

/* }}} */

  avw_read(argv[1],&im);
  avw_read(argv[2],&im2);

  xysize=im.x*im.y;
  size=xysize*im.z;

  out=malloc(sizeof(FDT)*size*2);

  for(z=0;z<im.z;z++)
    {
      for(i=0;i<xysize;i++)
	{
	  if (!invert)
	    {
	      out[z*2*xysize+i]=im.i[z*xysize+i];
	      out[(z*2+1)*xysize+i]=im2.i[z*xysize+i];
	    }
	  else
	    {
	      out[(im.z*2-1-z*2)*xysize+i]=im2.i[z*xysize+i];
	      out[(im.z*2-2-z*2)*xysize+i]=im.i[z*xysize+i];
	    }
	}
    }

  im.z*=2;
  im.i=out;

  im.thresh=im.min=im.max=0;
  find_thresholds(&im,0.1);

  avw_write(argv[3],im);
  return(0);
}

/* }}} */
