/*  avwmeants.cc

    Mark Jenkinson, FMRIB Image Analysis Group

    Copyright (C) 2001 University of Oxford  */

/*  CCOPYRIGHT  */


// Creates a mean time series (ignoring zeros) from the input 4D volume
// Saves the results as a column in a text file

#include "newimage/newimageall.h"
#include "miscmaths/miscmaths.h"

using namespace NEWIMAGE;
using namespace MISCMATHS;


void print_usage(const string& progname) {
  cerr << "Usage: " << progname << " <input 4D volume> <output text file> [<mask volume> [-v]] " 
       << endl << endl;
  cerr << "e.g.   " << progname << " filtered_func_data meants.txt my_mask" 
       << endl;
  cerr << "       " << progname << " filtered_func_data meants.txt" 
       << endl;
}



int main(int argc,char *argv[])
{

  Tracer tr("main");

  string progname=argv[0];
  if (argc<3) { 
    print_usage(progname);
    return -1; 
  }

  bool verbose = false;
  if (argc>=5) {
    string optarg=argv[4];
    if (optarg=="-v") { 
      verbose = true;
    } else {
      cerr << "Unrecognised option: " << optarg << endl;
    }
  }

  volume4D<float> vin;
  read_volume4D(vin,argv[1]);

  volume<float> mask;
  if (argc>=4) {
    read_volume(mask,argv[3]);
  } else {
    mask = vin[0];
    mask = 1.0;
  }

  if (!samesize(vin[0],mask)) {
    cerr << "ERROR: Mask and Input volumes have different (x,y,z) size." 
	 << endl;
    return -1;
  }

  int nt = vin.tsize();
  Matrix meants(nt,1);
  meants = 0;
  long int num=0;

  for (int z=mask.minz(); z<=mask.maxz(); z++) {
    for (int y=mask.miny(); y<=mask.maxy(); y++) {
      for (int x=mask.minx(); x<=mask.maxx(); x++) {
	if (fabs(mask(x,y,z))>1e-8) {  // arbitrary "0" threshold
	  meants += vin.voxelts(x,y,z);
	  num++;
	  if (verbose) {
	    cout << x << " " << y << " " << z << " " << vin.voxelts(x,y,z).t();
	  }
	}
      }
    }
  }

  cout << "Number of voxels used = " << num << endl;

  // normalise for number of valid entries
  meants /= (float) num;

  // save the result
  write_ascii_matrix(meants,argv[2]);

  return 0;
}

