/* {{{ Copyright etc. */

/*  avwroi.c - extract cuboid ROI from image

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2001 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwroi <input> <output> <xmin> <xsize> <ymin> <ysize> <zmin> <zsize>\n");
  printf("       avwroi <input> <output> <tmin> <tsize>\n");
  printf("       avwroi <input> <output> <xmin> <xsize> <ymin> <ysize> <zmin> <zsize> <tmin> <tsize>\n\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc,argv) 
     int argc;
     char **argv;
{
  FDT *out;
  image_struct im;
  int x,y,z,t,xmin=0,xsize=0,ymin=0,ysize=0,zmin=0,zsize=0,tmin=0,tsize=0,argindex;

  /* {{{ process args */

if (argc==1)
     usage();

avw_read(argv[1],&im);
im.t=MAX(im.t,1);

argindex=3;

if (argc==5)
{
  xmin=0;
  xsize=im.x;
  ymin=0;
  ysize=im.y;
  zmin=0;
  zsize=im.z;
  tmin=atoi(argv[argindex++]);
  tsize=atoi(argv[argindex++]);
}
else if (argc==9)
{
  xmin=atoi(argv[argindex++]);
  xsize=atoi(argv[argindex++]);
  ymin=atoi(argv[argindex++]);
  ysize=atoi(argv[argindex++]);
  zmin=atoi(argv[argindex++]);
  zsize=atoi(argv[argindex++]);
  tmin=0;
  tsize=im.t;
}
else if (argc==11)
{
  xmin=atoi(argv[argindex++]);
  xsize=atoi(argv[argindex++]);
  ymin=atoi(argv[argindex++]);
  ysize=atoi(argv[argindex++]);
  zmin=atoi(argv[argindex++]);
  zsize=atoi(argv[argindex++]);
  tmin=atoi(argv[argindex++]);
  tsize=atoi(argv[argindex++]);
}
else
  usage();

out=malloc(xsize*ysize*zsize*tsize*sizeof(FDT));  

/* }}} */
  /* {{{ process roi and output */

  for(t=0;t<tsize;t++)
    for(z=0;z<zsize;z++)
      for(y=0;y<ysize;y++)
	for(x=0;x<xsize;x++)
	  out[t*zsize*ysize*xsize + z*ysize*xsize + y*xsize + x] =
	    im.i[(t+tmin)*im.z*im.y*im.x + (z+zmin)*im.y*im.x + (y+ymin)*im.x + x+xmin];

  im.i=out;
  im.x=xsize;	  
  im.y=ysize;	  
  im.z=zsize;	  
  im.t=tsize;	  

  avw_write(argv[2],im);

/* }}} */

  exit(0);
}

/* }}} */
