
/*  avwsplit.c - split 4D Analyze file into 3D files for SPM

    David Flitney, FMRIB Image Analysis Group

    Copyright (C) 2000 University of Oxford  */

/*  CCOPYRIGHT */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "avwio/avwio.h"

int main(int argc, char **argv)
{
  AVW *src;
  short x, y, z, v, t;
  float vx, vy, vz, tr;
  size_t bufsz;
  void *buffer;
  unsigned int i;
  char units[1024];
  char filename[10000];

   if (argc<2)
    {
      printf("\nUsage: avwsplit <inputfile.hdr>\n\n");
      exit(1);
    }

  strcpy(filename,argv[1]);
  if ( (strcmp(".hdr",filename+strlen(filename)-4)==0) ||
       (strcmp(".img",filename+strlen(filename)-4)==0) )
    filename[strlen(filename)-4]=0;

  if((src=AvwOpen(filename,"r"))==NULL) {
    perror("avwsplit");
    exit(-1);
  }

  AvwGetDim(src,&x,&y,&z,&v);
  AvwGetVoxDim(src,&vx,&vy,&vz,&tr);
  AvwGetVoxUnits(src, units);

  bufsz = x * y * z * (AvwGetDataType(src, &t) / 8);
  buffer = (void *) calloc(bufsz, sizeof(char));

  for(i = 0; i < v; i++) {
    AVW *dest;
    char opfn[1024];

    AvwReadVolumes(src, buffer, 1);

    sprintf(opfn, "vol%04d", i);
    fprintf(stderr, "Writing %s\r", opfn);

    dest = AvwOpen(opfn, "w");
    AvwCloneHeader(dest, src);
    AvwSetDim(dest, x, y, z, 1);
    AvwWriteVolumes(dest, buffer, 1);
    AvwClose(dest);
  }

  fprintf(stderr, "\n");

  return(0);
}
