#{{{ copyright and setup

#   renderhighres - render stats onto high res structurals
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   TCLCOPYRIGHT

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

#}}}
#{{{ feat5:renderhighres

proc feat5:renderhighres { w } {
    global PXHOME FSLDIR VARS argc argv PWD entries rendervars

    #{{{ setup main window

toplevel $w

wm title      $w "Colour render high res FEAT stats"
wm iconname   $w "FEAT render"
wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

frame $w.f
pack $w.f -in $w -side top

FSLFileEntry $w.f.featdir \
	-variable entries($w,1) \
	-pattern "*.feat" \
	-directory "~" \
	-label "Select a FEAT directory" \
	-title "Select a FEAT directory" \
	-width 20 \
	-dirasfile "design.fsf" \
	-filterhist VARS(history)

frame $w.f.minmax
set rendervars(zmin) 1
tixControl $w.f.zmin -label "Min" -variable rendervars(zmin) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }
set rendervars(zmax) 15
tixControl $w.f.zmax -label "Max" -variable rendervars(zmax) -step 1 -min 0 -selectmode immediate -options { entry.width 4 }
pack $w.f.zmin $w.f.zmax -in $w.f.minmax -side left -anchor w -padx 5

frame $w.f.talairach
label $w.f.talairach.label -text "Do rendering in Talairach space instead of original high res space"
set rendervars(talairach) 0
checkbutton $w.f.talairach.yn -variable rendervars(talairach)
pack $w.f.talairach.label $w.f.talairach.yn -in $w.f.talairach -side left -anchor w

frame $w.f.keep
label $w.f.keep.label -text "Keep 3D files after generating 2D colour pictures"
set rendervars(keep) 0
checkbutton $w.f.keep.yn -variable rendervars(keep)
pack $w.f.keep.label $w.f.keep.yn -in $w.f.keep -side left -anchor w

pack $w.f.featdir $w.f.minmax $w.f.talairach $w.f.keep -in $w.f -padx 5 -pady 5 -anchor w

#}}}
    #{{{ Button Frame

frame $w.btns
frame $w.btns.b -relief raised -borderwidth 1
    
button $w.apply -command "feat5:renderhighres_proc $w" -text "Go"

button $w.cancel -command "destroy $w" -text "Exit"

pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
pack $w.apply $w.cancel -in $w.btns.b -side left -expand yes -padx 3 -pady 3 -fill y

pack $w.btns -in $w -side bottom -fill x

#}}}
}

#}}}
#{{{ feat5:renderhighres_proc

proc feat5:renderhighres_proc { w } {
    #{{{ setup

global FSLDIR entries rendervars

set FD $entries($w,1)

puts "Processing $FD"

set TAL ""
set KEEP ""
if { $rendervars(talairach) } {
    set TAL -tal
}
if { $rendervars(keep) } {
    set KEEP -keep
}

catch { exec sh -c "${FSLDIR}/bin/renderhighres $FD $rendervars(zmin) $rendervars(zmax) $TAL $KEEP" } errmsg
puts $errmsg

puts "Done"

#}}}
}

#}}}
#{{{ call GUI and wait

wm withdraw .
feat5:renderhighres .r
tkwait window .r

#}}}
