/*  BandPassOptions.cc

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  CCOPYRIGHT  */

#define WANT_STREAM
#define WANT_MATH

#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include "newmatap.h"
#include "newmatio.h"
#include "BandPassOptions.h"

using namespace NEWMAT;
namespace FILM {

BandPassOptions* BandPassOptions::gopt = NULL;

void BandPassOptions::parse_command_line(int argc, char** argv, ofstream& logfile)
{

  if(argc<2){
    print_usage(argc,argv);
    exit(1);
  }
  
  int inp = 1;
  int n=1;
  string arg;
  char first;
  
  logfile << "Arguments: " << endl;
  while (n<argc) {
    arg=argv[n];

    // output argument to logfile:
    logfile << arg << " ";

    if (arg.size()<1) { n++; continue; }
    first = arg[0];
    if (first!='-') {
      if(inp == 1)
	gopt->inputfname = arg;
      else if(inp == 2)
	gopt->lowcut = atoi(argv[n]);
      else if(inp == 3)
	gopt->highcut = atoi(argv[n]);
      else if(inp == 4)
	gopt->thresh = atoi(argv[n]);
      else
	{
	  cerr << "Mismatched argument " << arg << endl;
	  break;
	}
      n++;
      inp++;
      continue;
    }
    
    // put options without arguments here
    if ( arg == "-help" ) {
      print_usage(argc,argv);
      exit(0);
    } else if ( arg == "-v" ) {
      gopt->verbose = true;
      n++;
      continue;
    } 
    if (n+1>=argc) 
      { 
	cerr << "Lacking argument to option " << arg << endl;
	break; 
      }

    // put options with 1 argument here
    if ( arg == "-rn" ) {
      gopt->datadir = argv[n+1];      
      logfile << argv[n+1] << " ";
      n+=2;
    } else { 
      cerr << "Unrecognised option " << arg << endl;
      n++;
    } 
  }  // while (n<argc)

  logfile << endl << "---------------------------------------------" << endl << endl;

  if (gopt->inputfname.size()<1) {
    print_usage(argc,argv);
    throw Exception("Input filename not found");
  }
  if (gopt->thresh < 0) {
    print_usage(argc,argv);
    throw Exception("Invalid thresh");
  }
}

void BandPassOptions::print_usage(int argc, char *argv[])
{
  cout << "Usage: " << argv[0] << " [options] <groupfile> <lowcut> <highcut> <thresh>\n\n"
       << " Set highcut or low cut to zero if not needed.\n"
       << " Everything greater than highcut and lower than lowcut is removed.\n"
       << "  Available options are:\n"
       << "        -v                                 (outputs full data)\n" 
       << "        -rn <dir>                          (directory name to store results in, default is "
       << gopt->datadir << ")\n"
       << "        -help\n\n";
}

#ifndef NO_NAMESPACE
}
#endif
