/* $Id: readavw.c,v 1.3 2003/02/22 18:38:46 mark Exp $ */
#include <stdlib.h>
#include <limits.h>
#include <math.h>

#include "avwimage.h"
#include "imutils.h"

AVWImage *ReadAVW(const char *filename)
{
  AVW *avw;
  AVWImage *im;
  char *myfilename;

  if((im = (AVWImage *) calloc(1, sizeof(AVWImage))) == NULL)
    { perror("ReadAVW"); exit(-1); }

  if((myfilename = (char *)calloc(strlen(filename) + 1, sizeof(char))) == NULL)
    {
      perror("Allocating room for myfilename");
      exit(-1);
    }
  strcpy(myfilename, filename);

  if ( (strcmp(".hdr", myfilename + strlen(myfilename) - 4) == 0) ||
       (strcmp(".img", myfilename + strlen(myfilename) - 4) == 0) )
    myfilename[strlen(myfilename) - 4] = '\0';

  avw = AvwOpen(myfilename, "r");

  AvwGetDataType(avw, &(im->dt));

  switch(im->dt)
    {
    case DT_NONE:          
      im->bpv = 0;                     
      break;
    case DT_UNSIGNED_CHAR: 
      im->bpv = sizeof(unsigned char); 
      im->dtmin.asChar = 0;              
      im->dtmax.asChar = 255;
      break;
    case DT_SIGNED_SHORT:  
      im->bpv = sizeof(short);
      im->dtmin.asShort = SHRT_MIN; 
      im->dtmax.asShort = SHRT_MAX;
      break;
    case DT_SIGNED_INT:    
      im->bpv = sizeof(int);
      im->dtmin.asInt = INT_MIN; 
      im->dtmax.asInt = INT_MAX;
      break;
    case DT_FLOAT:         
      im->bpv = sizeof(float);
      im->dtmin.asFloat = -1e10; 
      im->dtmax.asFloat = 1e10;
      break;
    case DT_COMPLEX:       
      im->bpv = sizeof(COMPLEX);       
      im->dtmin.asComplex.real = -1e10; im->dtmin.asComplex.imag = 0; 
      im->dtmax.asComplex.real =  1e10; im->dtmax.asComplex.imag = 0;
      break;
    case DT_DOUBLE:        
      im->bpv = sizeof(double);        
      break;
    }

  AvwGetDim(avw, &(im->xdim), &(im->ydim), &(im->zdim), &(im->vdim));
  if(im->xdim == 0) im->xdim = 1;
  if(im->ydim == 0) im->ydim = 1;
  if(im->zdim == 0) im->zdim = 1;
  if(im->vdim == 0) im->vdim = 1;

  AvwGetVoxDim(avw, &(im->xvox), &(im->yvox), &(im->zvox), &(im->tr));
  if(im->xvox < 0.00001) im->xvox = 1;
  if(im->yvox < 0.00001) im->yvox = 1;
  if(im->zvox < 0.00001) im->zvox = 1;
  if(im->tr   < 0.00001) im->tr   = 1;
  im->xavox = fabs(im->xvox);
  im->yavox = fabs(im->yvox);
  im->zavox = fabs(im->zvox);
  
  AvwGetMinMax(avw, &(im->min), &(im->max));

  im->orient = avw->header.hist.orient;
  memcpy(im->study, avw->header.hk.db_name, 18);
  memcpy(im->descrip, avw->header.hist.descrip, 80);
  memcpy(im->scannum, avw->header.hist.scannum, 10);
  memcpy(im->date, avw->header.hist.exp_date, 10);
  memcpy(im->patient, avw->header.hist.patient_id, 10);

  {
    short short_array[3];
    memcpy(short_array,avw->header.hist.originator,3*sizeof(short));
    im->xo=short_array[0];
    im->yo=short_array[1];
    im->zo=short_array[2];
  }

  if((im->data = (void *) calloc(im->xdim * im->ydim * im->zdim * im->vdim, 
				 im->bpv)) == NULL)
    { perror("ReadAvw(data)"); exit(-1); }

  AvwReadVolumes(avw, im->data, im->vdim);

  if((im->header = (struct dsr *) calloc(1, sizeof(struct dsr))) == NULL)
    { perror("ReadAvw(avw)"); exit(-1); }

  memcpy(im->header, &avw->header, sizeof(struct dsr));

  AvwClose(avw);

  switch(im->orient)
    {
    case AVWIM_AXIAL:    /* Leave it axially sliced */ break;
    case AVWIM_SAGITTAL: reorder(im, sag2ax);          break;
    case AVWIM_CORONAL:  reorder(im, cor2ax);          break;
    default: break;
    }

  /* code to reorder data if negative voxel sizes ?????? */

  return im;
}

