/*
  Function to convert F statistic to a Z statistic
  Uses routines from libprob.a

  Stuart Clare, FMRIB Physics Group

  Copyright (c) 1998 University of Oxford
*/

/* CCOPYRIGHT */

#include <stdio.h>
#include <math.h>
#include "libcprob.h"
#include "ftoz.h"

#define TOL 10e-16

long ftoz(double *f,double *z,long n,double df1, double df2)
{
  long i;
  float logp,p;

  /* Calculate extrapolation coeficients */
  /*
  double F[6],Z[6],x;
  double SFZ,SF2,mF,mZ,m;

  x=incbi(df2/2,df1/2,TOL);
  F[0]=(df2*(1-x))/(df1*x);
  for(i=1;i<6;i++)F[i]=F[0]-(i*0.1);
  for(i=0;i<6;i++)Z[i]=ndtri(1-incbet(df2/2,df1/2,df2/(df2+(df1*F[i]))));
  for(i=0,mF=0,mZ=0;i<6;i++)mF+=F[i]/6;
  for(i=0,SFZ=0,SF2=0;i<6;i++){
    SFZ+=(F[i]-mF)*Z[i];
    SF2+=(F[i]-mF)*(F[i]-mF);
  }
  m=SFZ/SF2;
  */

  /* Do the conversion */
  for(i=0;i<n;i++){
    if(f[i]<1)z[i]=0;
    else {
      
      /* new version */
      if(!islargef(f[i],df1,df2,&logp)){
	p = fdtr(df1,df2,f[i]);
	z[i] = ndtri(p);
      } else {
	z[i] = logp2largez(logp);
      }

      /* old version
	p=incbet(df2/2,df1/2,df2/(df2+(df1*f[i])));
	if(p<TOL){
	z[i]=log(f[i]-F[0]+(1/m))+Z[0]-log(1/m);
	}
	else {
	if(p>1)z[i]=0;
	else z[i]=ndtri(1-p);
	}
      */
    }
  }
  return(0);
}

int islargef(float f, int d1, int d2, float* logp)
{
  if(f > 1.0 && d1>1){
    *logp=largef2logp(f,d1,d2);	
    return issmalllogp(*logp);
  }
  else return 0;
}

int issmalllogp(float logp)
{
  if(logp < -14.5) return(1);
  else return(0);
}

float largef2logp(float f, int d1, int d2)
{
    int N = 20,i;
    double alpha,m,n,loggam,top,bot,iter;
    float logp;


    if (f<=0.0) return 0.0;
    if (d1<=0 || d2<=0) return 0.0;

    alpha=d1/(double)d2;
    m=(d1+d2)/2.0;
    n=(1-d1/2.0);
    loggam = (d1/2.0)*log(d1/(double)d2)-logbeta(d2/2.0,d1/2.0);
    top = 1.0;
    bot = n+m-1;
    iter = 0.0;
    
    for(i=1;i<=N;i++){
      iter = iter + top*pow(f,(-(n+i-1)))/(pow(alpha,i)*bot);	
      top = top*(n-1+i)*(-1);
      bot = bot*(n+m-1+i);
    }

    logp = loggam-(m-1)*log(1+alpha*f)+log(iter);

    return logp;
}

float logp2largez(float logp) 
{
  int m;
  float pi,log2pi,z0,zn,b;

  pi = 3.141592653590;
  log2pi = log(2*pi);
  b = -2*logp - log2pi; 
  z0 = sqrt(b);
  zn = z0;
  for (m=1; m<=3; m++) {
    zn = sqrt(b + 2*log(((3/(zn*zn) - 1)/(zn*zn) + 1)/zn) );
  }
  return zn;
}

float logbeta(float v, float w)
{
  return lgam(v)+lgam(w)-lgam(v+w);
}
