
/*  libss.h - basic C include stuff

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  CCOPYRIGHT */

#include "libssbase.h"

typedef struct {
  FDT *i;                                    /* image data */
  int x, y, z, t,                            /* image dimensions */
    dt, bpv;                                 /* datatype and bytes-per-voxel */
  double xv, yv, zv,                         /* voxel size (always all +ve) */
    xv0, yv0, zv0,                           /* voxel size as set in original header */
    xo, yo, zo,                              /* origin */
    tr,                                      /* seconds between volumes */
    min, max, thresh2, thresh98, thresh,     /* various brightness thresholds */
    lthresh, uthresh,                        /* lower and upper thresholds - some procs will ignore data outside these */
    dtmin, dtmax;                            /* min and max values for data type */
  char orient,                               /* orientation */
    lut[24];                                 /* lut file */
  float info;                                /* slice order information */
} image_struct;  

FDT median(double, FDT*, int);
double dmedian(double, double*, int);
FDT mean(FDT*, int);
FDT TLI(image_struct, double, double, double);
int make_isometric (image_struct, image_struct*);
int p2c(int, int, int, double, double, double, double, double, double, double*, double*, double*);
FDT getp2c(image_struct, double, double, double, double, double, double);
int find_histogram (image_struct*, int*, int);
/*void find_roi_histogram (image_struct*, int, int, int, int, int, int, int*, int);*/
void find_thresholds (image_struct*, double);
void c_of_g (image_struct, double*, double*, double*);
void invert_y (image_struct);
double find_radius (image_struct, double);
void print_image_struct(image_struct);
void init_image_struct(image_struct*);
void tempfilt(image_struct, double, double, int);
void spatgauss(image_struct, FDT*, double, double, double);
void spatfilt(image_struct, double);
void spatmedian(image_struct im, int width);
void intnorm(image_struct, int, double);
void fmrimask(image_struct, image_struct);
void resample(image_struct, image_struct*, float);
void sample(image_struct, image_struct, float);
