/*  miscplot.cc  -- miscellaneous plotting functions

    Christian F. Beckmann, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2002 University of Oxford */

/*  CCOPYRIGHT */

#include <strstream>
#include <stdio.h>

#include "miscplot.h"
#include "miscmaths/miscprob.h"
#include "gdfonts.h"

extern "C" {
 #include "gd.h"
 #include "gdc.h"
 #include "gdchart.h"
}

namespace MISCPLOT{

//  MATLAB Line colors
unsigned long   sc[64] = {0x8080FF,0x008000,0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,0x404040,0x0000FF,
			  0x008000,0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,0x404040,0x0000FF,0x008000,
			  0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,0x404040,0x0000FF,0x008000,0xFF0000,
			  0x00BFBF,0xBF00BF,0xBFBF00,0x404040,0x0000FF,0x008000,0xFF0000,0x00BFBF,
			  0xBF00BF,0xBFBF00,0x404040,0x0000FF,0x008000,0xFF0000,0x00BFBF,0xBF00BF,
			  0xBFBF00,0x404040,0x0000FF,0x008000,0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,
			  0x404040,0x0000FF,0x008000,0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,0x404040,
			  0x0000FF,0x008000,0xFF0000,0x00BFBF,0xBF00BF,0xBFBF00,0x404040,0x0000FF};

using namespace NEWMAT;
using namespace MISCMATHS;


string int2str(int n)
  {
    ostrstream os;
    //    os.fill(' ');
    //    os.width(width);
    os.setf(ios::internal, ios::adjustfield);
    os << n << '\0';
    return os.str();
  }

string float2str(float f, int width, int prec, bool scientif)
  {
    ostrstream os;
    int redw = int(std::abs(std::log10(std::abs(f))))+1;
    if(width>0)
      os.width(width);
    if(scientif)
      os.setf(ios::scientific);
    os.precision(redw+std::abs(prec));
    os.setf(ios::internal, ios::adjustfield);
    os << f << '\0';
    return os.str();
  }

void miscplot::add_legend(void* ptr, unsigned long cmap[], bool inside){
  
  int lbllength = 0;
  for(int ctr= 0; ctr < int(labels.size()); ctr++)
    lbllength = max(lbllength,int(labels[ctr].length()));

  int linebrk = 5;
  int space = gdFontSmall->w + 2;
  int width = 15;

  int xsize = gdImagePtr(ptr)->sx;
  if(!inside)
    xsize += 2*space + width + lbllength * gdFontSmall->w;
  int ysize = gdImagePtr(ptr)->sy;
  
  gdImagePtr newim;
  newim = gdImageCreate(xsize,ysize);
  gdImageCopy(newim,gdImagePtr(ptr),0,0,0,0,gdImagePtr(ptr)->sx,gdImagePtr(ptr)->sy);

  int black = gdImageColorResolve(newim, 0, 0, 0);
  
  int ycoor = 2*+gdFontSmall->h;
  for(int ctr=0; ctr < int(labels.size()); ctr++){
    int xcoor = gdImagePtr(ptr)->sx;
    if(inside)
      xcoor -= 2*space + width + lbllength * gdFontSmall->w;

    int col =  gdImageColorResolve(newim, ((cmap[ctr]&PVRED)>>16),((cmap[ctr]&PVGRN)>>8), (cmap[ctr]&0x000000FF));

    gdImageLine(newim, xcoor, ycoor + gdFontSmall->h/2, xcoor + width, ycoor + gdFontSmall->h/2,col);
    gdImageLine(newim, xcoor, ycoor + gdFontSmall->h/2+1, xcoor + width, ycoor + gdFontSmall->h/2+1,col);
    char *s = (char*)labels[ctr].c_str();
    gdImageString(newim, gdFontSmall, xcoor + width + space, ycoor, (unsigned char*) s, black);
    ycoor += gdFontSmall->h + linebrk;
  }
  outim = newim;
}


void miscplot::timeseries(const Matrix& mat, string filename, string title, 
			  float tr, int ysize, int width, int prec, bool sci){
  int numlines=mat.Nrows();
  int numpoint=mat.Ncols();

  float* data = new float[numlines*numpoint];
 
  for(int ctr1=1;ctr1<=numlines; ctr1++){
    for(int ctr2=1;ctr2<=numpoint; ctr2++){
      data[(ctr1-1)*numpoint + ctr2-1] = mat(ctr1,ctr2);
    }
  }

  int spacing = 1;

  if(numpoint>20) spacing = 2;
  if(numpoint>40) spacing = 5;
  if(numpoint>100) spacing = 10;
  if(numpoint>500) spacing = 20;
  if(numpoint>1200) spacing = 50;
  //if(numpoint>2000) spacing = 100;

  char* ctl = new char[numpoint];
  if((tr == 0.0)&&(spacing%2 == 1) || (tr > 0.0))
    ctl[0] = TRUE;
  else
    ctl[0] = FALSE;
  for(int ctr=1;ctr<numpoint;ctr++){
    if ((((ctr+1)%spacing == 0)&&(tr == 0.0))
	||(((ctr+1)%spacing == 0)&&(tr != 0.0)))
      ctl[ctr]=TRUE;
    else
      ctl[ctr]=FALSE;
  } 
  
  char** lbls = new char*[numpoint];
 
  //vector<string> s(numpoint);
  string s;
  lbls[0] = new char[2];
  if(tr == 0.0)
    strcpy(lbls[0],string("1").c_str());
  else
    strcpy(lbls[0],string("0").c_str());
  
  for(int ctr=1;ctr<numpoint;ctr++){
    if(ctl[ctr]){
      if(tr == 0.0)
	s = int2str(ctr+1);
      else
	s = float2str((ctr)*tr,width,prec,sci);
      lbls[ctr] = new char[s.length()+1];
      strcpy(lbls[ctr],s.c_str());
    }
    else{
      lbls[ctr] = new char[1];
      strcpy(lbls[ctr],string("").c_str());
    }
  }


  {
    int tmp = _gdccfoo1;
    tmp++;
    long unsigned int tmp2 = _gdccfoo2;
    tmp2++;
  }

  GDC_xlabel_ctl = ctl;

  GDC_BGColor   = 0xFFFFFFL;                  /* backgound color (white) */
  GDC_LineColor = 0x000000L;                  /* line color      (black) */
  GDC_SetColor  = &(sc[0]);                   /* MATLAB-like colors */

  GDC_title = (char*)title.c_str();

  GDC_ticks = GDC_TICK_LABELS;
  GDC_grid  = GDC_TICK_NONE;
  GDC_yaxis = TRUE;
  GDC_yaxis2 = FALSE;
  GDC_xaxis = TRUE;
  GDC_xaxis_angle = 0;
  GDC_ylabel_density = 50;
  GDC_ylabel_fmt = "%.2f";
  GDC_title_size = GDC_SMALL;

  int i,j;
  GDC_requested_ymax = 1.10*mat.Maximum2(i,j);
  GDC_requested_ymin = 1.10*mat.Minimum2(i,j);

  int xsize = std::max(4*numpoint,750);
  if(xsize>1200) xsize=(int)floor(std::max(xsize/2.0,750.0));

  //GDC_hard_size = TRUE;
  //GDC_hard_xorig = 50;
  //GDC_hard_graphwidth = xsize - 65;

  if(filename.substr(filename.size()-4,filename.size())!=string(".png"))
    filename += string(".png");
  
  FILE  *outpng1 = fopen(filename.c_str(), "wb" );
  GDC_image_type     = GDC_PNG;

  if (labels.size()>0)
    GDC_hold_img = GDC_EXPOSE_IMAGE;
  
  GDC_out_graph( xsize, ysize, outpng1 , GDC_LINE, numpoint, (char**) lbls , numlines, data, NULL ); 
  //GDC_out_graph( xsize, ysize, outpng1 , GDC_LINE, numpoint, NULL , numlines, data, NULL );
  fclose( outpng1 );
  
  if (labels.size()>0){
    outpng1 = fopen(filename.c_str(), "wb" );
    add_legend(GDC_image, sc);
    
    gdImagePng(outim, outpng1); 
    fclose( outpng1 );
    GDC_destroy_image(GDC_image);
    if(outim) gdImageDestroy(outim);
  }

  for(int ctr=0;ctr<numpoint;ctr++){
    delete [] lbls[ctr];
  }
    
  delete [] data;
  delete [] ctl;
  delete [] lbls;
}



void miscplot::histogram(const Matrix& mat, string filename, string title){

  RowVector datam = mat.Row(1);
  int numpoint=datam.Ncols();
  int i,j;
  float tmax = datam.Maximum2(i,j);
  float tmin = datam.Minimum2(i,j); 
  float trange = tmax-tmin;
  int bins = (int)floor(MISCMATHS::sqrt(numpoint));
  float intsize = trange / bins;
  int xlint = (int)ceil(trange/6);
  int binperint = (int)ceil(xlint / intsize);

  intsize = float(xlint) / float(binperint);

  // float xmin = ceil(std::abs(1.02*tmin)/intsize)*intsize * sign(tmin) + intsize/2;
//   float xmax = ceil(std::abs(1.02*tmax)/intsize)*intsize * sign(tmax) + intsize/2;
  
//   bins = (int)((xmax-xmin)/intsize);

//   cerr <<" before hist " << endl;
//   Histogram hist(datam.t(), xmin-0.5*intsize, xmax+0.5*intsize, bins+1);
//   cerr << " betweeb hist " << endl;
//   hist.generate();
//   cerr << " after generate " << endl;
//   Matrix bindata;
//   bindata = hist.getData().t();

  float xmin = ceil(std::abs(1.02*tmin)/intsize)*intsize * sign(tmin);
  float xmax = ceil(std::abs(1.02*tmax)/intsize)*intsize * sign(tmax);

  bins = (int)((xmax-xmin)/intsize);

  Matrix bindata(1,bins);
  bindata = 0.0;

  double binsize = (xmax-xmin)/std::max(bins,1);
  for(int ctr = 1; ctr<= datam.Ncols(); ctr++){
    bindata(1,std::max(std::min(int(floor((datam(ctr) - xmin) / binsize) + 1),bins),1))++;
  }
  //cerr << " calculated bindata " << endl;

  int factor = 1;
  numpoint = factor * bindata.Ncols();
  float* data = new float[numpoint]; 
  for(int ctr1=0;ctr1< numpoint; ctr1++){
      data[ctr1] = bindata(1,(int)floor(float(ctr1 / factor + 1)));
  }
  
  RowVector xax(numpoint);
  for(int ctr1=0;ctr1< numpoint; ctr1++){
    xax(ctr1+1) = xmin + (ctr1+ 0.5) * intsize; 
  }

  char* ctl = new char[numpoint];
  xlint = (int)ceil((xmax-xmin)/9);
  
  ctl[0]=FALSE;
  for(int ctr=1;ctr<numpoint;ctr++){
    int lblpoint = (int)(MISCMATHS::round(abs(xax(ctr))/xlint)*xlint);
    if(xax(ctr)<0) lblpoint *= -1;
     if( (xax(ctr)<lblpoint)&&(xax(ctr+1)>lblpoint) ){
       ctl[ctr]=TRUE;
     }
     else
       ctl[ctr]=FALSE;
  } 

  char** lbls = new char*[numpoint];
 
  string s;

  for(int ctr=0;ctr<numpoint;ctr++){
    if(ctl[ctr]){
      s = float2str((int)MISCMATHS::round(xax(ctr+1)/xlint)*xlint,3,2,false);
      lbls[ctr] = new char[s.length()+1];
      strcpy(lbls[ctr],s.c_str());
    }
    else{
      lbls[ctr] = new char[1];
      strcpy(lbls[ctr],string("").c_str());
    }
  }
  
 
  GDC_xlabel_ctl = ctl;

  GDC_BGColor   = 0xFFFFFFL;                  /* backgound color (white) */
  GDC_LineColor = 0x000000L;                  /* line color      (black) */
  GDC_SetColor  = &(sc[0]);                   /* MATLAB-like colors */

  GDC_title = (char*)title.c_str();
  GDC_title_size = GDC_SMALL;

  GDC_ticks = GDC_TICK_LABELS;
  GDC_grid  = GDC_TICK_NONE;
  GDC_yaxis = FALSE;
  GDC_yaxis2 = FALSE;
  GDC_xaxis = TRUE;
  GDC_xaxis_angle = 0;
  GDC_bar_width =  75;
  GDC_ylabel_density = 50;
  GDC_requested_ymax = 1.15*bindata.Maximum2(i,j);
  GDC_requested_ymin = 1.15*bindata.Minimum2(i,j);

  //int xsize = std::max(4*numpoint,750);
  //  if(xsize>1200) xsize=(int)floor(std::max(xsize/2.0,750.0));
  int xsize = 600;
  int ysize = 400;

  if(filename.substr(filename.size()-4,filename.size())!=string(".png"))
    filename += string(".png");

  FILE  *outpng1 = fopen(filename.c_str(), "wb" );
  GDC_image_type     = GDC_PNG;
  GDC_out_graph( xsize, ysize, outpng1 , GDC_BAR, numpoint, (char**) lbls , 1, data, NULL ); 
  fclose( outpng1 );

  for(int ctr=0;ctr<numpoint;ctr++){
    delete [] lbls[ctr];
  }
    
  delete [] data;
  delete [] lbls;
  delete [] ctl;
}

void miscplot::gmmfit(const Matrix& mat, Matrix& mu, Matrix& sig, 
		      Matrix& pi, string filename, string title, 
		      bool gammamix, float meanoffset, float detailfactor)
{

  int numlines=mu.Ncols()+1;

  RowVector datam = mat.Row(1);
  int numpoint=datam.Ncols();
  int i,j;
  float tmax = datam.Maximum2(i,j);
  float tmin = datam.Minimum2(i,j); 
  float trange = tmax-tmin;
  int bins = (int)floor(MISCMATHS::sqrt(numpoint));
  float intsize = trange / bins;
  int xlint = (int)ceil(trange/6);
  int binperint = (int)ceil(xlint / intsize);

  intsize = float(xlint) / float(binperint);

  float xmin = ceil(std::abs(1.02*tmin)/intsize)*intsize * sign(tmin);
  float xmax = ceil(std::abs(1.02*tmax)/intsize)*intsize * sign(tmax);

  bins = (int)((xmax-xmin)/intsize);

  Matrix bindata(1,bins);
  bindata = 0.0;

  intsize = (xmax-xmin)/std::max(bins,1);

  for(int ctr = 1; ctr<= datam.Ncols(); ctr++){
    bindata(1,std::max(std::min(int(floor((datam(ctr) - xmin) / intsize) + 1),bins),1))++;
  }
  
  int factor = 9;
  // cerr << " bindata.SumAbsoluteValue() " << bindata.SumAbsoluteValue() << endl;
  bindata = bindata / max(double(factor*bindata.SumAbsoluteValue()),double(1.0));

  numpoint = factor * bindata.Ncols();
  float* histdata = new float[numpoint]; 
  for(int ctr1=0;ctr1< numpoint; ctr1++){
      histdata[ctr1] = bindata(1,(int)floor(float(ctr1 / factor + 1)));
  }

  float offset = 0.0*intsize;

  intsize = intsize / std::max(factor,1);
  RowVector xax(numpoint);
  for(int ctr1=0;ctr1< numpoint; ctr1++){
    xax(ctr1+1) = xmin + (ctr1) * intsize + offset; 
  }

  for(int ctr1=1;ctr1< sig.Ncols(); ctr1++)
    if(sig(1,ctr1)<0.001){
      sig(1,ctr1) = 0.001;
      pi(1,ctr1) = 0.0;
    }

  Matrix fit;
  fit = SP(normpdf(xax,mu,sig),pi.t()*ones(1,numpoint));

  //if(sig(1,1)>0.001)
  //  fit.Row(1) = pi(1,1) * normpdf(xax, mu(1,1),sig(1,1));

  if(gammamix){
    if(pi(1,2)>0.0001)
      fit.Row(2)  = pi(1,2)*gammapdf( xax - 0.0*(mu(1,1) + meanoffset),
			      mu(1,2), sig(1,2));
    if(pi(1,3)>0.0001)
    fit.Row(3)  = pi(1,3)*gammapdf(-xax + 0.0* (mu(1,1) - meanoffset),
			   std::abs(mu(1,3)), sig(1,3));
  }
  //  else{
  //  if((pi(1,2)>0.001)&&(sig(1,2)>0.001))
  //    fit.Row(2) = pi(1,2) * normpdf(xax, mu(1,2),sig(1,2));
  //   if((pi(1,3)>0.001)&&(sig(1,3)>0.001))
  //    fit.Row(3) = pi(1,3) * normpdf(xax, mu(1,3),sig(1,3));  
  // }
     
  fit = sum(fit,1) & fit;
  fit = fit / fit.Row(1).SumAbsoluteValue();

  float* data = new float[(numlines)*numpoint];
  for(int ctr1=1;ctr1<=numlines; ctr1++){
    for(int ctr2=1;ctr2<=numpoint; ctr2++){
      data[(ctr1-1)*numpoint + ctr2-1] = fit(ctr1,ctr2);
    }
  }					   
  
  char* ctl = new char[numpoint];
  xlint = (int)ceil((xmax-xmin)/9);
  
  ctl[0]=FALSE;
  for(int ctr=1;ctr<numpoint;ctr++){
    int lblpoint = (int)(MISCMATHS::round(abs(xax(ctr))/xlint)*xlint);
    if(xax(ctr)<0) lblpoint *= -1;
     if( (xax(ctr)<lblpoint)&&(xax(ctr+1)>lblpoint) ){
       ctl[ctr]=TRUE;
     }
     else
       ctl[ctr]=FALSE;
  } 
  
  char** lbls = new char*[numpoint];
 
  //vector<string> s(numpoint);
  string s;

  for(int ctr=0;ctr<numpoint;ctr++){
    if(ctl[ctr]){
      //s[ctr]= float2str((int)MISCMATHS::round(xax(ctr+1)/xlint)*xlint,3,2,false);
      //  lbls[ctr] = s[ctr].c_str();
      s = float2str((int)MISCMATHS::round(xax(ctr+1)/xlint)*xlint,3,2,false);
      lbls[ctr] = new char[s.length()+1];
      strcpy(lbls[ctr],s.c_str());
    }
    else{
      lbls[ctr] = new char[1];
      strcpy(lbls[ctr],string("").c_str());
    }
  }

  GDC_xlabel_ctl = ctl;

  unsigned long*   sc2 = new unsigned long[numlines];
  for(int ctr=0;ctr<numlines;ctr++) sc2[ctr] = 0xFFDD00;
  sc2[0]=0xFF0000;
  if(gammamix)
    sc2[1]=0x00aa00;

  unsigned long  sc3[64];
  for(int ctr=0;ctr<64;ctr++) sc3[ctr] = sc[ctr];
  for(int ctr=0;ctr<numlines;ctr++){
    sc3[ctr+1] = sc2[ctr];
  }

  GDC_BGColor   = 0xFFFFFFL;                  /* backgound color (white) */
  GDC_LineColor = 0x000000L;                  /* line color      (black) */
  GDC_SetColor  = &(sc2[0]);                  /* MATLAB-like colors */

  GDC_title = (char*)title.c_str();
  GDC_title_size = GDC_SMALL;

  GDC_ticks = GDC_TICK_LABELS;
  GDC_grid  = GDC_TICK_NONE;
  GDC_yaxis = FALSE;
  GDC_yaxis2 = FALSE;

  GDC_xaxis = TRUE;
  GDC_xaxis_angle = 0;
  GDC_bar_width = 100;
  GDC_ylabel_density = 50;
  GDC_requested_ymax = 1.05*std::max(bindata.Maximum2(i,j),fit.Maximum2(i,j));
  GDC_requested_ymin = float(0.0);

  //int xsize = std::max(4*numpoint,750);
  //  if(xsize>1200) xsize=(int)floor(std::max(xsize/2.0,750.0));
  int xsize = 600;
  int ysize = 400;

  if(filename.substr(filename.size()-4,filename.size())==string(".png"))
    filename = filename.substr(0,filename.size()-4);

  FILE  *outpng1 = fopen((filename.append(".png")).c_str(), "wb" );
  GDC_image_type     = GDC_PNG;

  if (labels.size()>0)
    GDC_hold_img = GDC_EXPOSE_IMAGE;

  GDC_out_graph( xsize, ysize, outpng1 , GDC_COMBO_LINE_AREA, numpoint, (char**) lbls , numlines, data, histdata);  
  //GDC_out_graph( xsize, ysize, outpng1 , GDC_COMBO_LINE_AREA, numpoint, NULL, numlines, data, histdata); 
  fclose( outpng1 );
  
  if (labels.size()>0){
    outpng1 = fopen(filename.c_str(), "wb" );
    add_legend(GDC_image, sc3,TRUE);
    
    gdImagePng(outim, outpng1); 
    fclose( outpng1 );
    GDC_destroy_image(GDC_image);
    if(outim) gdImageDestroy(outim);
  }

  if((detailfactor > 0.0)&&(fit.Nrows()>2)){
  
    Matrix tmp(1,fit.Ncols()-2);
    tmp=sum(fit.Rows(3,fit.Nrows()),1);
   
    float req_max = detailfactor*tmp.Maximum2(i,j);

    if(req_max<0.0000001)
      req_max= 1.05*std::max(bindata.Maximum2(i,j),fit.Maximum2(i,j));
    
    for(int ctr1=1;ctr1<=numlines; ctr1++){
      for(int ctr2=1;ctr2<=numpoint; ctr2++){
	data[(ctr1-1)*numpoint + ctr2-1] = std::min(req_max,data[(ctr1-1)*numpoint + ctr2-1]);
      }
    }	

    for(int ctr1=0;ctr1< numpoint; ctr1++){
     histdata[ctr1] =  std::min(req_max, histdata[ctr1]);
    }

    GDC_requested_ymax = req_max;
    GDC_requested_ymin = float(0.0);
    GDC_title = (char*)title.append("  (detail)").c_str();
    GDC_title_size = GDC_SMALL;

    int xsize = 600;
    int ysize = 400;

    if(filename.substr(filename.size()-4,filename.size())==string(".png"))
      filename = filename.substr(0,filename.size()-4);

    FILE  *outpng1 = fopen((filename.append("_detail.png")).c_str(), "wb" );
    GDC_image_type = GDC_PNG;
    
    if (labels.size()>0)
      GDC_hold_img = GDC_EXPOSE_IMAGE;
    
    GDC_out_graph( xsize, ysize, outpng1 , GDC_COMBO_LINE_BAR, numpoint, (char**) lbls , numlines, data, histdata); 
    //GDC_out_graph( xsize, ysize, outpng1 , GDC_COMBO_LINE_BAR, numpoint, NULL , numlines, data, histdata); 
    fclose( outpng1 );
    
    if (labels.size()>0){
      outpng1 = fopen(filename.c_str(), "wb" );
      add_legend(GDC_image, sc3,TRUE);
      
      gdImagePng(outim, outpng1); 
      fclose( outpng1 );
      GDC_destroy_image(GDC_image);
      if(outim) gdImageDestroy(outim);
    }
  }

  for(int ctr=0;ctr<numpoint;ctr++){
    delete [] lbls[ctr];
  }

  delete [] histdata;
  delete [] data;
  delete [] lbls;
  delete [] ctl;
  delete [] sc2;
  }

}
//int main(int argc, char *argv[])
//{
//  Matrix tmp(3,5);
//  tmp = 0.0;
//  plot_timeseries(tmp,"miscplottest.png","Hi");
//  exit(0);
//}
