/*  displacement.cc - Artificial Motion Generator
    
    Peter Bannister, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2000 University of Oxford  */

/*  CCOPYRIGHT */


#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <strstream>

#include "newimage/newimageall.h"
#include "miscmaths/optimise.h"
#include "newimage/costfns.h"
#include "newmatap.h"

#include "Globaloptions.h"
#include "Log.h"

using namespace MISCMATHS;
using namespace NEWMAT;
using namespace NEWIMAGE;
using namespace UTILS;

int main (int argc,char** argv){
  
  volume4D<float> timeseries;
  volume<float> vout;
  Matrix aff(4,4);
  int no_volumes;
  string inputfname;
  string inputbasename = "/usr/people/prb/medx/motion/synthetic/";
  Log& logger = Log::getInstance();
  logger.setDir("/usr/people/prb/medx/motion/synthetic/params");

  // repeat this block 4 times for start and end of both activation levels
  inputfname = inputbasename + "null/start";
  cerr << "Reading stationary time series" << inputfname << " ... " << endl;
  volumeinfo vinfo;
  read_volume4D(timeseries, inputfname, vinfo);
  no_volumes = timeseries.tsize();  
  vout = timeseries[0];
  
  vout.setextrapolationmethod(extraslice);
  vout.setinterpolationmethod(trilinear);

 
  for (int i = 0; i < no_volumes; i++) {
    
    char strc[50];
    ostrstream osc(strc,50);
    osc << "mat_"<< i+100 << '\0';
    cerr << "Applying matrix: " << strc << endl;
    logger.in(strc, aff,4,4);
    cerr << "read in matrix" << endl;
    cerr << aff << endl;
    affine_transform(vout,timeseries[i], aff);
    cerr << "transform applied ... " << endl;
    timeseries[i] = vout;
  }
  
  cerr << "Saving displaced time series" + inputfname + " ... "<< endl;
  save_volume4D_dtype(timeseries, inputfname, dtype(inputfname), vinfo);
  
  
}
