/*  test.cc Evaluates RMSdiff between matricies
    
    Peter Bannister, FMRIB Image Analysis Group
    
    Copyright (C) 1999-2001 University of Oxford  */

/*  CCOPYRIGHT */


#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <strstream>
#include <iomanip>

#include "newmatap.h"
#include "miscmaths/miscmaths.h"
#include "newimage/newimageall.h"

#include "Globaloptions.h"
#include "Log.h"

using namespace MISCMATHS;
using namespace NEWMAT;
using namespace NEWIMAGE;
using namespace UTILS;
using namespace MISCMATHS;

int main (int argc,char** argv){
  
  volume4D<float> timeseries;
  volume<float> vout;
  volumeinfo vinfo;
  Matrix displacement[200], correction[200];
  ColumnVector cvec(3);
  cvec = 0.0;
  int no_volumes;
  float rmax=80.0;
  string inputfname;
  string inputbasename = "/usr/people/prb/medx/motion/";
  string directory;
  ColumnVector vtemp(6);
  ColumnVector centre(3);
  centre = 0;
  Log& logger = Log::getInstance();

  if (argc < 3){
    cerr << "Usage: testprog <reference_mats_dir> <test_mats_dir>" << endl;
    exit(1);
  }

  inputfname = "/usr/people/prb/medx/motion/validation/fmri_3.2/fmri_3.2.hdr";
  
  cerr << "Reading stationary time series" << inputfname << " ... " << endl;
  read_volume4D(timeseries,inputfname,vinfo);
  no_volumes = timeseries.tsize();

  cerr << no_volumes << endl;
  
  vout = timeseries[0];
  
  centre(1) = 0.5*(vout.xsize() - 1.0)*vout.xdim();
  centre(2) = 0.5*(vout.ysize() - 1.0)*vout.ydim();
  centre(3) = 0.5*(vout.zsize() - 1.0)*vout.zdim();

  directory = argv[1];
  logger.setDir(directory);

  cerr << directory << endl;

  for (int i = 0; i < no_volumes; i++) {
    
    char strc[50];
    ostrstream osc(strc,50);
    osc << "MAT_"<< setw(4) << setfill('0') << i << '\0';
    cerr << "Reading matrix: " << strc << endl;
    logger.in(strc, displacement[i],4,4);
    cerr << "read in matrix" << endl;
    cerr << displacement[i] << endl;
  }

  /*
   logger.setDir("/usr/people/prb/medx/motion/roberto/group2/subj9/mats");
   for (int i = 0; i < no_volumes; i++) {
    
     char strc[50];
     ostrstream osc(strc,50);
     if (i < 10)
       osc << "MAT_000" << i << '\0';
     else if (i < 100)
       osc << "MAT_00" << i << '\0';
     else if (i < 1000)
       osc << "MAT_0" << i << '\0';
     cerr << "Reading matrix: " << strc << endl;
     logger.in(strc, displacement[i],4,4);
     cerr << "read in matrix" << endl;
     cerr << displacement[i] << endl;
   }
  */

  /*
   logger.setDir("/usr/people/prb/medx/motion/sinctest/spm/");
  
   for (int i = 0; i < no_volumes; i++) {
     char strc[50];
     ostrstream osc(strc,50);
     osc << "spm_mat_corrected_"<< i << '\0';
     cerr << "Reading matrix: " << strc << endl;
     logger.in(strc, correction[i],4,4);
     cerr << "read in matrix" << endl;
     cerr << correction[i] << endl;
   } 
  */

  /*
  logger.setDir("/usr/people/prb/medx/motion/validation/reference/air/");
  
  
  for (int i = 0; i < no_volumes; i++) {
     char strc[50];
     ostrstream osc(strc,50);
     osc << "newairmat_"<< i+1  << '\0';
     cerr << "Reading matrix: " << strc << endl;
     logger.in(strc, correction[i], 4, 4);
     cerr << "read in matrix" << endl;
     cerr << correction[i] << endl;
  } 
  */
 
 
  directory = argv[2];
  logger.setDir(directory);
  
  /*
  for (int i = 0; i < no_volumes; i++) {
    char strc[50];
    ostrstream osc(strc,50);
    osc << "spm_mat_corrected_"<< setw(4) << setfill('0') << i << '\0';
    cerr << "Reading matrix: " << strc << endl;
    logger.in(strc, correction[i], 4, 4);
    cerr << "read in matrix" << endl;
    cerr << correction[i] << endl;
  }
  */
  
  /*
  for (int i = 0; i < 10; i++) {
    char strc[50];
    ostrstream osc(strc,50);
    osc << "mat_000"<< i << '\0';
    cerr << "Reading matrix: " << strc << endl;
    logger.in(strc, correction[i], 4, 4);
    cerr << "read in matrix" << endl;
    cerr << correction[i] << endl;
  }

  for (int i = 10; i < no_volumes; i++) {
    char strc[50];
    ostrstream osc(strc,50);
    osc << "mat_00"<< i << '\0';
    cerr << "Reading matrix: " << strc << endl;
    logger.in(strc, correction[i], 4, 4);
    cerr << "read in matrix" << endl;
    cerr << correction[i] << endl;
  }
  */

   for (int i = 0; i < no_volumes; i++) {    
     char strc[50];
     ostrstream osc(strc,50);
     osc << "MAT_"<< setw(4) << setfill('0') << i << '\0';
     cerr << "Reading matrix: " << strc << endl;
     logger.in(strc, correction[i], 4, 4);
     cerr << "read in matrix" << endl;
     cerr << correction[i] << endl;
   }


    for (int i = 0; i < no_volumes; i++) {
      //float rms = rms_deviation(Identity(4),displacement[no_volumes/2]*displacement[i].i(),centre,rmax);// ref
      float rms = rms_deviation(correction[i],displacement[no_volumes/2]*displacement[i].i(),centre,rmax);// middle ref 
      //float rms = rms_deviation(correction[i], displacement[0]*displacement[i].i(),centre,rmax); // first ref
      cout << rms << endl;
  } 
 
}























