#

#   fsl:exec execute a shell command with logging all setup
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 2000-2003 University of Oxford
#
#   TCLCOPYRIGHT

#
# usage: fsl:echo <filename> "string" [-n]
# -n : overwrite instead of appending
#
proc fsl:echo { thefilename thestring args } {
    if { $thefilename != "" } {
	if { [ llength $args ] != 0 } {
	    catch { exec sh -c "echo \"$thestring\" >  $thefilename" >& /dev/null } putserr
	} else {
	    catch { exec sh -c "echo \"$thestring\" >> $thefilename" >& /dev/null } putserr
	}
    } else {
	catch { puts $thestring } putserr
    }
}


#
# usage: fsl:exec "the command" [-n] [-r]
# -n : don't output logging information
# -r : run remotely on a machine setup by CLUSTERRSH
#
proc fsl:exec { thecommand args } {

    global logout comout CLUSTERRSH FSLDIR FD

    # process args
    set do_logout 1
    set do_rsh 0
    for { set argindex 0 } { $argindex < [ llength $args ] } { incr argindex 1 } {
	set thearg [ lindex $args $argindex ]

	if { $thearg == "-r" && $CLUSTERRSH != "" } {
	    set do_rsh 1
	}

	if { $thearg == "-n" } {
	    set do_logout 0
	}
    }

    # save just the command to report.com if it has been setup
    if { [ info exists comout ] && $comout != "" } {
	fsl:echo $comout "$thecommand\n"
    }

    # save command rootname for using in "ps" commands
    if { [ info exists FD ] } {
	fsl:echo ${FD}/.fslcurrent [ file tail [ lindex $thecommand 0 ] ] -n
    }

    # if logout doesn't exist, set empty so that logging will go to screen
    if { ! [ info exists logout ] } {
	set logout ""
    }

    # run and log the actual command
    if { $do_logout } {
	fsl:echo $logout "\n$thecommand"
    }
    if { $do_rsh } {
	set thecommand "$CLUSTERRSH \"cd [ pwd ] ; $thecommand\""
    }
    if { $do_logout } {
	if { $logout != "" } {
	    catch { exec sh -c $thecommand >>& $logout } errmsg
	    set errmsg [ exec sh -c "tail -1 $logout" ]
	} else {
	    catch { exec sh -c $thecommand } errmsg
	    catch { puts $errmsg } putserr
	}
    } else {
	catch { exec sh -c $thecommand } errmsg
    }

    # now return errmsg in case the exec call needs to know the output
    return $errmsg
}

