/* {{{ Copyright etc. */

/*  slicer - produce any slicings of a 3D image

    Stephen Smith & Christian Beckmann, FMRIB Image Analysis Group

    Copyright (C) 1999-2002 University of Oxford  */

/*  CCOPYRIGHT  */

/* }}} */
/* {{{ defines, includes and typedefs */
 
#include "libvis/miscpic.h"
//#include "fmribmain.h"
 
using namespace NEWIMAGE;
using namespace MISCPIC;

/* }}} */
/* {{{ usage */


void usage(void)
{
  printf("\nUsage: slicer <input> [input2] [main options] [output options - any number of these]\n\n");

  printf("Main options: [-l <lut>] [-s <scale>] [-i <intensitymin> <intensitymax>] [-t]\n");
  printf("These must be before output options and must be in the above order.\n");
  printf("-l <lut> : use a different colour map from that specified in the header.\n");
  printf("-t       : produce semi-transparent (dithered) edges.\n\n");
  printf("-n       : use nearest-neighbour interpolation for output.\n\n");

  printf("Output options:\n");
  printf("[-x/y/z <slice> <filename>]      : output sagittal, coronal or axial slice\n     (if <slice> >0 it is a fraction of image dimension, if <0, it is an absolute slice number)\n");
  printf("[-a <filename>]                  : output mid-sagittal, -coronal and -axial slices into one image\n");
  printf("[-A <width> <filename>]          : output _all_ axial slices into one image of _max_ width <width>\n");
  printf("[-S <sample> <width> <filename>] : as -A but only include every <sample>'th slice\n\n");

  exit(1);
}

/* }}} */
/* {{{ main */

//template <class T>
int fmrib_main(int argc, char* argv[])
{
  volumeinfo vol1info;
  //volume<T> vol1, vol2(1,1,1);

  volume<float> vol1, vol2(1,1,1);

  read_volume(vol1,string(argv[1]),vol1info);

  int i = 2;
  if ( (argc>i) && (argv[i][0]!='-') ){
    read_volume(vol2,string(argv[i])); i++;
  }
  
  bool dbgflag = FALSE;
  if ( (argc>i) && (string(argv[i]) == string("-d")) ){
    dbgflag = TRUE; i++;
  }

  char tmp[10000];
  sprintf(tmp," ");
  for(;i<argc;i++){
    strcat(strcat(tmp,argv[i])," ");}
  
  //miscpic<T> newpic;
  miscpic newpic;

  return newpic.slicer(vol1, vol2, tmp, &vol1info, dbgflag);
}

int main(int argc,char *argv[])
{
  if (argc<2) usage();
  
  //  return call_fmrib_main(dtype(std::string(argv[1])),argc,argv);  
  //return call_fmrib_main(DT_FLOAT,argc,argv);  
  return fmrib_main(argc,argv); 

}

/* }}} */
