#{{{ copyright and setup 

#   FEAT TCL functions library
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2003 University of Oxford
#
#   TCLCOPYRIGHT

#}}}
#{{{ feat5:scrollform_resize

proc feat5:scrollform_resize { w0 viewport } {

    set MAXWIDTH  950
    set MAXHEIGHT 700

    set fwidth  [ winfo width $viewport.f ]
    if { $fwidth > $MAXWIDTH } {
	set fwidth $MAXWIDTH
	pack $w0.xsbar -fill x -in $w0
    } else {
	pack forget $w0.f.xsbar
    }

    set fheight [ winfo height $viewport.f ]
    if { $fheight > $MAXHEIGHT } {
	set fheight $MAXHEIGHT
	pack $w0.f.ysbar -side right -fill y -in $w0.f
    } else {
	pack forget $w0.f.ysbar
    }

    $viewport configure -width $fwidth -height $fheight -scrollregion [ $viewport bbox all ]
}

#}}}
#{{{ feat5:multiple_select

proc feat5:multiple_select { w which_files windowtitle } {
    global FSLDIR fmri VARS PWD feat_files unwarp_files initial_highres_files highres_files

    if { $fmri(multiple) < 1 && $which_files == 0 } {
	SelectSinglePage:Dialog $w fmri(groupname) "feat5:updateselect $w 0"	 
    } else {
	#{{{ setup window

    set count 0
    set w0 ".dialog[incr count]"
    while { [ winfo exists $w0 ] } {
        set w0 ".dialog[incr count]"
    }

    toplevel $w0

    wm iconname $w0 "Select"
    wm iconbitmap $w0 @${FSLDIR}/tcl/fmrib.xbm

    wm title $w0 $windowtitle

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top

    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set"
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat5:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

#}}}
	#{{{ setup file selections

for { set i 1 } { $i <= $fmri(multiple) } { incr i 1 } {

    if { $which_files < 1 } {

	if { ( $fmri(level) == 1 && $fmri(analysis) != 4 && $fmri(analysis) != 0 ) || $fmri(inputtype) == 2 } {
	    FSLFileEntry $w0.filename$i -label " $i:   " -directory $PWD -title "Select input data" -width 60 -filterhist VARS(history) \
		    -variable feat_files($i) -pattern "*.hdr"
	} else {
	    FSLFileEntry $w0.filename$i -label " $i:   " -directory $PWD -title "Select input data" -width 60 -filterhist VARS(history) \
		    -variable feat_files($i) -pattern "*.feat" -dirasfile "design.fsf"
	}

    } elseif { $which_files == 1 } {
	FSLFileEntry $w0.filename$i -label " $i:   " -directory $PWD -title "Select input data" -width 60 -filterhist VARS(history) \
		-variable unwarp_files($i) -pattern "*.hdr"
    } elseif { $which_files == 2 } {
	FSLFileEntry $w0.filename$i -label " $i:   " -directory $PWD -title "Select input data" -width 60 -filterhist VARS(history) \
		-variable initial_highres_files($i) -pattern "*.hdr"
    } elseif { $which_files == 3 } {
	FSLFileEntry $w0.filename$i -label " $i:   " -directory $PWD -title "Select input data" -width 60 -filterhist VARS(history) \
		-variable highres_files($i) -pattern "*.hdr"
    }

    pack $w0.filename$i -in $w0.f.viewport.f -side top -expand yes -fill both -padx 3 -pady 3
}

#}}}
	#{{{ setup buttons

frame $w0.btns
frame $w0.btns.b -relief raised -borderwidth 1

button $w0.cancel -command "feat5:multiple_check $w $which_files 1 1 0; destroy $w0" -text "OK"

pack $w0.btns.b -side bottom -fill x -padx 3 -pady 5
pack $w0.cancel -in $w0.btns.b -side left -expand yes -padx 3 -pady 3 -fill y
pack $w0.btns -expand yes -fill x

#}}}
    }
}

#}}}
#{{{ feat5:multiple_check

proc feat5:multiple_check { w which_files load updateimageinfo dummy } {
    global FSLDIR fmri feat_files unwarp_files initial_highres_files highres_files

    if { $which_files < 0 } {
	set load 0
	featquery_whichstats $w
    }

    if { $which_files > 0 } {

	if { !$fmri(reg_yn) } {
	    return 0
	}

	if { [ exec sh -c "${FSLDIR}/bin/avwnvols [ file rootname $fmri(regstandard) ] 2> /dev/null" ] < 1 } {
	    MxPause "Warning: standard image not valid"
	    return 1
	}
    }

    if { $fmri(multiple) < 2 } {
	set nmultiple 1
    } else {
	set nmultiple $fmri(multiple)
    }

    set AllOK 1

    for { set i 1 } { $i <= $nmultiple } { incr i 1 } {

	if { $which_files == 0 } {

	    if { ! [ info exists feat_files($i) ] || ! [ file exists $feat_files($i) ] } {
		set AllOK 0
	    } else {
		if { $fmri(level) == 1 && $fmri(analysis) != 4 && $fmri(analysis) != 0 } {
		    if {  $updateimageinfo } {
			feat5:updateimageinfo $w $i
		    }
		} else {

		    if { $fmri(inputtype) == 1 } {
			if { ! [ file exists $feat_files($i)/design.fsf ] } {
			    set AllOK 0
			} else {
			    if { $load && $i == 1 && $fmri(level)==1 } {
				feat5:load $w 0 $feat_files(1)/design.fsf
				MxPause "Warning: have just loaded in design information from the design.fsf
in the first FEAT directory in the list."
			    }
			}
		    }
		}
	    }

	    if { $fmri(level) == 2 && $AllOK } {
		feat5:updateselect $w 0
	    }

	} elseif { $which_files == 1 } {

	    if { ![ info exists unwarp_files($i) ] || [ exec sh -c "${FSLDIR}/bin/avwnvols [ file rootname $unwarp_files($i) ] 2> /dev/null" ] < 1 } {
		set AllOK 0
	    }

	} elseif { $which_files == 2 } {

	    if { ![ info exists initial_highres_files($i) ] || [ exec sh -c "${FSLDIR}/bin/avwnvols [ file rootname $initial_highres_files($i) ] 2> /dev/null" ] < 1 } {
		set AllOK 0
	    }

	} elseif { $which_files == 3 } {

	    if { ![ info exists highres_files($i) ] || [ exec sh -c "${FSLDIR}/bin/avwnvols [ file rootname $highres_files($i) ] 2> /dev/null" ] < 1 } {
		set AllOK 0
	    }

	}
    }
    
    if { ! $AllOK} {
	MxPause "Warning: you haven't filled in all the relevant selections with valid
filenames - this could include image data, FEAT directories,
unwarp input files and high-resolution images."
	return 1
    }

    return 0
}

#}}}

