#{{{ copyright and setup

#   featwatcher - GUI for watching FEAT analysis progress
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 2003 University of Oxford
#
#   TCLCOPYRIGHT

source [ file dirname [ info script ] ]/fslstart.tcl

#}}}
#{{{ featwatcher

proc featwatcher { w } {
    global fmri FSLDIR OSFLAVOUR argc argv

    toplevel $w
    wm title      $w "FEAT Watcher"
    wm iconname   $w "FEAT Watcher"
    wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm
#   wm geometry $w 800x500
    set fmri(font) [ tix option get font ]

    #{{{ process argument

if { [ lindex $argv 0 ] == "" } {
    puts "Usage: Featwatcher <feat_directory.feat>"
    exit 1
}

set FD [ lindex $argv 0 ]

if { ( $OSFLAVOUR != "cygwin" && [ string first / $FD ] != 0 ) ||
     ( $OSFLAVOUR == "cygwin" && [ string first : $FD ] != 1 ) } {
    set FD [ pwd ]/$FD
}

cd $FD

#}}}
    #{{{ FEAT directory name

frame $w.featdir -relief raised -borderwidth 1

label $w.featdir.label -text "$FD"

pack $w.featdir.label -in $w.featdir -padx 5 -pady 5 -side left -expand yes

#}}}
    #{{{ current command

frame $w.current -relief raised -borderwidth 1

message $w.current.label -text "" -font $fmri(font) -width 780

pack $w.current.label -in $w.current -padx 5 -pady 5 -side left -expand yes

#}}}
    #{{{ log

frame $w.log -relief raised -borderwidth 1

scrollbar $w.log.sbar -command "$w.log.text yview"
text $w.log.text -yscrollcommand "$w.log.sbar set"

pack $w.log.sbar -side right -fill y
pack $w.log.text -side left -expand yes -fill both

#}}}
    #{{{ bottom buttons

frame $w.btns -relief raised -borderwidth 1

set helpfile ${FD}/report.html
if { [ string first "C:/cygwin" $helpfile ] == 0 } {
    set helpfile [ string range $helpfile 9 end ]
}
button $w.btns.reportview   -text "View FEAT report webpage" -command "FmribWebHelp file: $helpfile"

button $w.btns.cancel -text "Exit Featwatcher (FEAT will continue)" -command "exit"

button $w.btns.help   -text "Help" -command "FmribWebHelp file: ${FSLDIR}/doc/feat5/featwatcher.html"

pack $w.btns.reportview $w.btns.cancel $w.btns.help -in $w.btns -padx 5 -pady 5 -side left -expand yes

#}}}

    pack $w.featdir $w.current $w.log -in $w -padx 5 -pady 5 -fill x 
    pack $w.btns -in $w -side bottom -padx 5 -pady 5 -fill x 

    set i 40
    while { 1 } {

	if { $i == 80 } {
	    if { ( $OSFLAVOUR == "unix" &&
		   [ catch { exec sh -c "ps o comm,pcpu,cputime,vsz | grep `cat .fslcurrent` | awk '{print \$1 \"   %cpu=\"\$2 \"   cputime=\"\$3 \"   mem=\"\$4}' " } fmricurrent ] == 0 ) ||
		 ( $OSFLAVOUR == "macos" &&
		   [ catch { exec sh -c "ps co command,pcpu,cputime,vsz | grep `cat .fslcurrent` | awk '{print \$1 \"   %cpu=\"\$2 \"   cputime=\"\$3 \"   mem=\"\$4}' " } fmricurrent ] == 0 ) ||
		 ( $OSFLAVOUR == "cygwin" &&
		   [ catch { exec sh -c "top n 1 b | grep `cat .fslcurrent` | awk '{print \$12 \"   %cpu=\"\$9 \"   cputime=\"\$11 \"   mem=\"\$6}' " } fmricurrent ] == 0 ) } {
		$w.current.label configure -text $fmricurrent
	    } else {
		$w.current.label configure -text ""
	    }

	    catch { exec sh -c "cat report.log" } fmrilog
	    $w.log.text delete 0.0 end
	    $w.log.text insert end $fmrilog
	    $w.log.text see end

	    set i 0
	}
	
	update
	incr i 1
	after 250
    }
}

#}}}
#{{{ call GUI and wait

if { ! [ info exists INGUI ] } {
    wm withdraw .
    featwatcher .r
    tkwait window .r
}

#}}}
