#

#{{{ setup

#   snr - measure SNR on an FMRI data set
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   TCLCOPYRIGHT

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

#}}}

set Norm   [ YesNoWidget "Carry out intensity normalization?" "Yes" "No" ]
set IgnoreEdge [ YesNoWidget "Ignore noise at volume edges? (You probably don't want to ignore the edge-related noise if you are interested in comparing motion correction methods)" "Yes" "No" ]

#{{{ get group and create new group

MxGetCurrentGroup OrigGroup

MxGetPagesFromGroup $OrigGroup Pages

# prepare list of volumes for new group
set NewList ""

set i 1
foreach Page $Pages {
    if { $i > 2 } {
	lappend NewList $Page
    }
    incr i 1
}

MxGroupVolumes $NewList "SubGroup" Group

#}}}
#{{{ do norm if required

if { $Norm } {
    MxIntensityNormalization $Group R 10000 10
}

#}}}
#{{{ create stats images

MxWithinGrpStatistics $Group M GroupMean
name_page $GroupMean "Mean"

MxDupPage $GroupMean Mask
name_page $Mask "Mask"

MxDupPage $GroupMean Div
name_page $Div "StdDev / Mean"

MxWithinGrpStatistics $Group S GroupStdDev
name_page $GroupStdDev "StdDev"

#}}}
#{{{ find image dimensions

MxGetImageProperties $GroupMean pl
set ww [ expr [ keylget pl Width  ] - 4 ]
set hh [ expr [ keylget pl Height ] - 4 ]
set ss [ keylget pl Slices ]

#}}}
#{{{ draw box in GroupMean

# need the MxSetCurrentPage commands because MxDrawBox doesn't work properly
MxSetCurrentPage $GroupMean
MxDrawBox $GroupMean   2 2 0 $ww $hh $ss GroupMeanBox

MxQuietStatistics $GroupMeanBox { Maximum Minimum } false Stats
set Min [ keylget Stats Minimum ]
set Max [ keylget Stats Maximum ]
set Thresh [ expr ( ( $Max - $Min ) / 10.0 ) + $Min ]
puts "Threshold = $Thresh"

#}}}
#{{{ create mask image by thresholding and eroding GroupMean

MxThreshold $Mask $Thresh B1

if { $IgnoreEdge } {
    if { $ss < 8 } {
	MxSetImageOpMode $Mask 2DAll
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/circle.ker
    } else {
	MxSetImageOpMode $Mask 3D
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/sphere333.ker
	MxBinaryMorphologyErode $Mask ${PXHOME}/filters/morphology/sphere333.ker
    }
} else {
    if { $ss < 8 } {
	MxSetImageOpMode $Mask 2DAll
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/circle.ker
    } else {
	MxSetImageOpMode $Mask 3D
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/sphere333.ker
	MxBinaryMorphologyDilate $Mask ${PXHOME}/filters/morphology/sphere333.ker
    }
}

MxSetImageOpMode $Mask 3D
MxThreshold $Mask 0.01 B1

#}}}
#{{{ create masked div and find final stats

MxExpr $Div = " 10000 * $Mask * ( $GroupStdDev / $GroupMean ) "

MxSetCurrentPage $Div
MxDrawBox $Div 2 2 0 $ww $hh $ss DivBox
MxQuietStatistics $DivBox { Mean Median } false 0.0001 Stats
set DivMean [ expr [keylget Stats Mean] / 100.0 ]
set DivMedian [ expr [keylget Stats Median] / 100.0 ]

#}}}

MxSetCurrentPage $OrigGroup

puts "Mean (StdDev Image / Mean Image)=$DivMean %\nMedian (StdDev Image / Mean Image)=$DivMedian %"

MxPause "Mean (StdDev Image/\nMean Image)=$DivMean%\n\nMedian (StdDev Image/\nMean Image)=$DivMedian%"

MxDeleteGroupPages { $Group } No
