#

#{{{ setup

#   susan.tcl - GUI for susan smoothing
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   TCLCOPYRIGHT

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

#}}}

#{{{ proc susan

proc susan { w } {

    global susanvars usanentries FSLDIR PWD INMEDX argc argv TN HOME
 
    #{{{ setup main window

toplevel $w

wm title $w "SUSAN   Structure-Preserving Noise Reduction"
wm iconname $w "SUSAN"
wm iconbitmap $w @$FSLDIR/tcl/fmrib.xbm

frame $w.f

#}}}
    #{{{ setup variable defaults

set susanvars($w,dt)  3
set susanvars($w,um)  1
set susanvars($w,dim) 3D

if { $INMEDX } {

    MxGetCurrentImage susanvars($w,inputP)

    set TN [ exec sh -c "${FSLDIR}/bin/tmpnam ${HOME}/.susan" ]

    set susanvars($w,input)  $TN
    set susanvars($w,output) $TN

    set type [ what_type 0 ]
    if { $type >1 && $type < 4 } {
	FSLSaveAs $susanvars($w,inputP) AVW $susanvars($w,input).hdr true
	susan:setrange $w dummy
    }
}

#}}}
    #{{{ input and output images (if not in medx)

if { ! $INMEDX } {

    if { $argc > 0 && [ string length [ lindex $argv 0 ] ] > 0 } {
	set inputname [ file rootname [ lindex $argv 0 ] ].hdr
	if { [ string first / $inputname ] == 0 || [ string first ~ $inputname ] == 0 } {
	    set susanvars($w,input) $inputname
	} else {
	    set susanvars($w,input) ${PWD}/$inputname
	}
	set susanvars($w,output) [ file rootname $susanvars($w,input) ]_susan.hdr
	susan:setrange $w dummy
    }

    FSLFileEntry $w.f.input \
	    -variable susanvars($w,input) \
	    -pattern "*.hdr" \
	    -directory $PWD \
	    -label "Input image   " \
	    -title "Select the input image" \
	    -width 55 \
	    -command "susan:setrange $w" \
	    -filterhist VARS(history)

    FSLFileEntry $w.f.output \
	    -variable susanvars($w,output) \
	    -pattern "*.hdr" \
	    -directory $PWD \
	    -label "Output image" \
	    -title "Select the output image" \
	    -width 55 \
	    -filterhist VARS(history)

    pack $w.f.input $w.f.output -in $w.f -side top -padx 5 -pady 5 -anchor w
}

#}}}
    #{{{ top row

    frame $w.f.top

    tixOptionMenu $w.f.op -label "Dimensionality" 
    $w.f.op add command 2D -label 2D
    $w.f.op add command 3D -label 3D
    $w.f.op config -variable susanvars($w,dim) 

    tixControl $w.f.bt -label "Brightness threshold" -variable susanvars($w,bt) -step 1 -min 0 -selectmode immediate -options {
	entry.width 8
    }

    tixControl $w.f.dt -label "Mask SD" -variable susanvars($w,dt) -step 1 -min 0 -selectmode immediate -options {
	entry.width 5
    }

    pack $w.f.top -in $w.f -side top -padx 3 -pady 3 -expand yes -anchor w

    pack $w.f.op $w.f.bt $w.f.dt -in $w.f.top -side left -padx 5 -pady 3 -expand yes -anchor w

#}}}
    #{{{ help text

label $w.f.label -wraplength 550 -text "SUSAN filtering works on 16 bit signed integer data.\nMask SD (approx HWHM) is in current length units (e.g. mm), not voxels.\nSet Mask SD to 0 for a (much faster) flat 3x3 voxels or 3x3x3 voxels mask." -fg "#303030"

pack $w.f.label -in $w.f -side top -anchor s

#}}}
    #{{{ advanced options

collapsible frame $w.f.opts -title "Advanced options"

#{{{ use median?

frame $w.f.um

label $w.f.um.label -text "Use median when no neighbourhood is found "

checkbutton $w.f.um.but -variable susanvars($w,um)

pack $w.f.um.label $w.f.um.but -in $w.f.um -side left

pack $w.f.um  -in $w.f.opts.b -anchor w -pady 5

#}}}
#{{{ more USANs

set susanvars($w,maxusans) 2
set susanvars($w,usans) 0

tixControl $w.f.usans -label "Separate images to find USAN from " \
	-variable susanvars($w,usans) -step 1 -min 0 -max $susanvars($w,maxusans) -selectmode immediate -command "susan:updateusan $w"

pack $w.f.usans -in $w.f.opts.b -anchor w -pady 5

set i 1
while { $i <= $susanvars($w,maxusans) } {
    frame $w.f.usanentries($i)

    if { $INMEDX } {

	frame $w.f.ue$i
	label $w.f.ue${i}.label -width 13 -text "USAN image $i"
	entry $w.f.ue${i}.inputvar -textvariable usanentries($w,$i) -width 20
	button $w.f.ue${i}.inputsel -text "Select" -command "SelectPage:Dialog $w $i 0 2 usanentries"
	pack $w.f.ue${i}.label $w.f.ue${i}.inputvar $w.f.ue${i}.inputsel -in $w.f.ue$i -padx 3 -pady 0 -side left

    } else {

	FSLFileEntry $w.f.ue$i \
		-variable usanentries($w,$i) \
		-pattern "*.hdr" \
		-directory $PWD \
		-label "USAN image $i" \
		-title "Select the USAN image" \
		-width 40 \
		-filterhist VARS(history)
	
    }

    set susanvars($w,ubt,$i) 1
    tixControl $w.f.ubt$i -label "Brightness threshold" -variable susanvars($w,ubt,$i) -step 1 -min 0 -selectmode immediate -options {
	entry.width 5
    }

    pack $w.f.ue$i $w.f.ubt$i -in $w.f.usanentries($i) -padx 3 -pady 3 -side left
    incr i 1
}

#}}}

pack $w.f.opts -in $w.f -side bottom -anchor w -pady 5

#}}}
    #{{{ button frame

    frame $w.btns
    frame $w.btns.b -relief raised -borderwidth 1
    
    button $w.ok -text "OK" -width 5 -command "susan:apply $w destroy"
    bind $w.ok <Return> {
        [winfo toplevel %W].ok invoke
    }
    
    button $w.apply -command "susan:apply $w keep" -text "Apply" -width 5
    bind $w.apply <Return> {
	[winfo toplevel %W].apply invoke
    }
	    
    button $w.cancel    -command "susan:destroy $w" \
	    -text "Cancel" -width 5
    bind $w.cancel <Return> {
	[winfo toplevel %W].cancel invoke
    }

    button $w.help -command "FmribWebHelp file: $FSLDIR/doc/susan/index.html" \
            -text "Help" -width 5
    bind $w.help <Return> {
        [winfo toplevel %W].help invoke
    }

    pack $w.btns.b -side bottom -fill x -padx 3 -pady 3
    pack $w.ok $w.apply $w.cancel $w.help -in $w.btns.b \
	    -side left -expand yes -padx 3 -pady 3 -fill y

#}}}
		
    pack $w.f $w.btns -expand yes -fill both
}

#}}}
#{{{ proc susan:setrange

proc susan:setrange { w dummy } {

    global susanvars FSLDIR INMEDX
    
    if { ! [ catch { exec sh -c "${FSLDIR}/bin/avwstats $susanvars($w,input) -r" } minmax ] } {
	set min [ lindex $minmax 0 ]
	set max [ lindex $minmax 1 ]
	set susanvars($w,bt) [ expr ( $max - $min ) / 10.0 ]
    }

    if { !$INMEDX && [ string length $susanvars($w,output) ] == 0 } {
	set susanvars($w,output) [ file rootname $susanvars($w,input) ]_susan.hdr
    }
}

#}}}
#{{{ proc susan:apply

proc susan:apply { w dialog } {

    global susanvars usanentries INMEDX TN

    foreach v { bt dt } {
	$w.f.$v update
    }

    if { $INMEDX } {
	#{{{ find and test current image

    set the_type [ what_type 1 ]
    if { $the_type < 2 || $the_type > 3 } {
	MxPause "You do not have a 2D or 3D image currently selected"
	return
    }

    MxGetCurrentFolder Folder
    MxGetCurrentImage susanvars($w,inputP)
    FSLSaveAs $susanvars($w,inputP) AVW $susanvars($w,input).hdr true

    set depth 1
    MxGetDimensions $susanvars($w,inputP) width height depth
    MxGetImageProperties $susanvars($w,inputP) pl
    set bpp  [ keylget pl Bits ]
    set enc  [ keylget pl Encoding ]
    set susanvars($w,name) [ keylget pl Name ]
    if { [ string compare ${bpp}${enc} 8UI ] != 0 && [ string compare ${bpp}${enc} 16SI ] != 0 } {
	MxPause "SUSAN can only process 8 bit unsigned or 16 bit signed data"
	return
    }

#}}}
	#{{{ process USAN entries

set usanlist ""

set i 1
while { $i <= $susanvars($w,usans) } {
    set success [ MxGetPageByName $Folder $usanentries($w,$i) usanpageP($i) ]

    if { $success } {
	MxPause "Not all of the USAN images are valid"
	return
    }

    FSLSaveAs $usanpageP($i) AVW ${TN}_usan${i}.hdr true
    set usanpage($i) ${TN}_usan${i}

    set udepth 1
    MxGetDimensions $usanpageP($i) uwidth uheight udepth

    if { $udepth != $depth || $uwidth != $width || $uheight != $height } {
	MxPause "Not all of the USAN images are of the same size as the input image"
	return
    }

    lappend usanlist $usanpage($i)
    puts "USAN $i Name=$usanentries($w,$i) Page=$usanpageP($i) File=$usanpage($i)"

    lappend usanlist $susanvars($w,ubt,$i)

    incr i 1
}

MxSetCurrentPage $susanvars($w,inputP)

#}}}
    } else {
	#{{{ process USAN entries

set usanlist ""

set susanvars($w,name) grot

set i 1
while { $i <= $susanvars($w,usans) } {
    lappend usanlist $usanentries($w,$i)
    lappend usanlist $susanvars($w,ubt,$i)
    incr i 1
}

#}}}
    }
    
    susan_proc $susanvars($w,input) $susanvars($w,name) $susanvars($w,bt) $susanvars($w,output) $susanvars($w,dt) $susanvars($w,dim) $susanvars($w,um) $susanvars($w,usans) $usanlist
    
    update idletasks
    
    if {$dialog == "destroy"} {
	susan:destroy $w
    }
}

#}}}
#{{{ proc susan:destroy

proc susan:destroy { w } {
    destroy $w
}

#}}}
#{{{ updates

proc susan:updateusan { w val } {
    global susanvars

    if { $susanvars($w,usans) == 0 } {
	pack $w.f.bt -in $w.f.top -before $w.f.dt -side left -padx 5 -pady 3 -expand yes -anchor w
    } else {
	pack forget $w.f.bt
    }

    set i 1
    while { $i <= $susanvars($w,maxusans) } {
	pack forget $w.f.usanentries($i)
	incr i 1
    }

    set i 1
    while { $i <= $susanvars($w,usans) } {
	pack $w.f.usanentries($i) -in $w.f.opts.b -anchor w -pady 5
	incr i 1
    }
}

#}}}

#{{{ tail end

wm withdraw .
susan .rename
tkwait window .rename

#}}}
