#

#{{{ copyright etc.

#   susan_proc.tcl - TCL call for susan smoothing
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   TCLCOPYRIGHT

#}}}

proc susan_proc { input name bt output dt dim um usans usanlist } {

    global FSLDIR INMEDX CLUSTERRSH

    #{{{ prepare thecommand

set thecommand "$CLUSTERRSH ${FSLDIR}/bin/susan_smooth $input $bt $output $dt $dim $um $usans"

set i 1
while { $i <= $usans } {
    set thecommand "$thecommand [ lindex $usanlist [ expr ( $i - 1 ) * 2 ] ] [ lindex $usanlist [ expr ( ( $i - 1 ) * 2 ) + 1 ] ]"
    incr i 1
}

if { $usans > 0 } {
    set thecommand "$thecommand ${input}_usan_size"
}

#}}}
    #{{{ call program

if { $INMEDX } {
    ScriptUpdate "Calling $thecommand\nAs this process is not fully integrated into MEDx, the traffic lights will not show up red during processing. Wait for this popup to disappear before continuing."
}

puts $thecommand

if { [ catch { exec sh -c $thecommand } ErrMsg ] != 0 } {
    puts "$ErrMsg"
}

puts Finished

if { $INMEDX } {
    CancelScriptUpdate
}

#}}}
    #{{{ load stuff into MEDx

if { $INMEDX } {

    MxGetCurrentFolder Folder

    if { $usans > 0 } {
        MxOpenImage $Folder ${input}_usan_size.hdr US
        MxGetImageProperties $US USProps
        keylset USProps Name "USAN size for SUSAN smoothed ${name}"
        MxSetImageProperties $US $USProps
	MxComputeDisplayRange $US umin umax
    }

    MxOpenImage $Folder ${output}.hdr SUSAN
    MxGetImageProperties $SUSAN SUSANProps
    keylset SUSANProps Name "SUSAN smoothed ${name}"
    MxSetImageProperties $SUSAN $SUSANProps
    MxComputeDisplayRange $SUSAN umin umax

    exec sh -c "rm -f ${input}*"
}

#}}}
}
